package Modules::ImageRotate;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------
# ImageRotate : tool to animate/rotate through the images in a library
#
# Options:
#
# id=N			library id of photo album, OR
# name=ABC		name of photo album
# interval=x	interval at which Images rotate
# format=onload|fade	onload:only rotate images on page load, not on an interval
#						fade: fade images in
#
# TODO: 
# abilty to link images
# sliding images on interval rotate
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Input;
use ExSite::Misc;
use ExSite::Content;
use Modules::BaseDCD;

# inherit basic behaviours from BaseDCD

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

# provide our own write() method

sub write {
    my ($this,$options) = @_;
    my $in = new ExSite::Input;
    my $q = $in->query;
    my %opt = &DecodeString($options);
    my %allopt = (%opt,%$q);
	
	$allopt{interval} = $allopt{interval} || 5000;
    
	my $db = $share{DB};
    my ($album,$album_id);
    if ($allopt{id}) {
        # album passed by id#
        $album_id = $allopt{id};
        $album = $db->fetch("page",$album_id);
    }
    elsif ($allopt{name}) {
        # album passed by name
        my @lib = $db->fetch_match("page",{section_id=>$share{Page}->id("section"),
                                           filename=>$allopt{name}});
        $album = $lib[0];
        $album_id = $album->{page_id};
    }
    if (!$album) {
        # look for "images" library in current site
        my @lib = $db->fetch_match("page",{section_id=>$share{Page}->id("section"),
                                           filename=>"images"});
        $album = $lib[0];
        $album_id = $album->{page_id};
    }

    if ($album) { 
		my @pics = $db->fetch_child("content", "page", $album_id, "name");
		my @image_urls;
		my @links;
		foreach my $p (@pics) {
			my $c = new ExSite::Content(content=>$p);
			$c->set_context("child");
			my $link = $c->get("content","description");
			$link = "\"" . $link . "\""; 			
			$link = &html_to_text($link);			
			my $url = $c->get_url;
			my $url = "\"" . $url . "\""; 
			push(@image_urls, $url);
			push(@links,$link);
		}
		my $imgs = join(',',@image_urls);
		my $links = join(',',@links);
		
		my $is_onload = 0;
		my $is_fade = 0;
		my $is_linked = 0;				
		if ( $allopt{format} =~ /onload/ ) { $is_onload = 1; }
		if ( $allopt{format} =~ /fade/ ) { $is_fade = 1; }
		if ( $allopt{format} =~ /linked/ ) { $is_linked = 1; }
		my $out = <<"DONE";

<script language="JavaScript">

<!-- fade functions start -->

function initImage() {
	imageId = 'adBanner1';
	image = document.getElementById(imageId);
	image.style.visibility = 'visible';
	setOpacity(image, 0);
	fadeIn(imageId,0);
}

function fadeIn(objId,opacity) {
	if (document.getElementById) {
		obj = document.getElementById(objId);
		if (opacity <= 100) {
			setOpacity(obj, opacity);
			opacity += 10;
			window.setTimeout("fadeIn('"+objId+"',"+opacity+")", 110);
		}
	}
}
function setOpacity(obj, opacity) {
	opacity = (opacity == 100)?99.999:opacity;
	// IE/Win
	obj.style.filter = "alpha(opacity:"+opacity+")";
	// Safari<1.2, Konqueror
	obj.style.KHTMLOpacity = opacity/100-.001;
	// Older Mozilla and Firefox
	obj.style.MozOpacity = (opacity/100)-.001;
	// Safari 1.2, newer Firefox and Mozilla, CSS3
	obj.style.opacity = opacity/100;
}

<!-- fade functions end -->

function get_random()
{
	var today = new Date()
	var seed = today.getSeconds()
    var ranNum= Math.floor(Math.random(seed)*imgCt1)
    return ranNum;
}	

var imgs1 = new Array($imgs)
if ($is_linked) {
	var links1 = new Array($links)
}
var imgCt1 = imgs1.length
var currentAd1 = get_random()

var postimages=new Array() //preload images
for (p=0;p<imgCt1;p++) {
	postimages[p]=new Image()
	postimages[p].src=imgs1[p]
}

function cycle1() {
	var image = document.getElementById('adBanner1')
	if ( $is_fade ) {
		image.style.visibility = 'hidden';
		setOpacity(image, 0);
	}
	window.setTimeout("cycle2()",100);
}

function cycle2() {
	currentAd1++
	if (currentAd1 == imgCt1) {
		currentAd1 = 0
	}	
	var image = document.getElementById('adBanner1')
	image.src = imgs1[currentAd1]
	if ($is_linked) {
		var image_link = document.getElementById('rotateLink')
		image_link.href = links1[currentAd1]
	}
	if ( $is_fade ) {
		initImage()
	}
}

if ($is_linked) { 
	var currentLink = links1[currentAd1];
	var html_linkopen = '<a id="rotateLink" target="_blank" href="' + currentLink + '">';
	var html_linkclose = '</a>';
}
else { 
	var currentLink = "#";
	var html_linkopen = '';
	var html_linkclose = '';
}

if (!$is_onload) {
	window.setInterval("cycle1()",$allopt{interval})
	document.write('<div class="ImageRotateHolder">' + html_linkopen + '<img class="ImageRotate" id="adBanner1" border="0" src="' + imgs1[currentAd1] + '" />' + html_linkclose + '</div>')  
}
else {
	document.write('<div class="ImageRotateHolder">' + html_linkopen + '<img class="ImageRotate" border="0" src="' + imgs1[currentAd1] + '" />' + html_linkclose + '</div>')
}
</script>
DONE
	}
    else {
		my $message = "photo album not found ($options)";
		$this->error($message);
		return "<p class=\"error\">$message</p>\n";
	}	
}

sub ioctl {
    my $this = shift;
    $_ = shift;
    if (/ModuleName/) {
        return "ImageRotate";
    }
    elsif (/ModuleInfo/) {
        return "The ImageRotate tool allows you to 
rotate through your image library.  It also allows you to 
fade images in and load random images on refresh.";
    }
}

sub html_to_text {
    $_ = shift;
    s/\r//g;  # remove linebreaks
    s/\n//g;  # remove linebreaks	
    return $_;
}

1;
