#!/usr/bin/perl
package Modules::PhotoForum;

#==========================================================================
# PhotoForum : a tool to share and comment on photos provided by users
#
# Uses the Zine framework to handle indexing, archiving, publishing, etc.
# We simply override a few admin/access control functions to allow anyone
# to post to the photoblog.
#==========================================================================


use strict;
use ExSite::Config;
use ExSite::Report;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Mail;
use ExSite::HTML;
use ExSite::Crypt;
use ExSite::Image;
use Modules::Zine;

use vars qw(@ISA);
@ISA = qw(Modules::Zine);

sub ioctl {
    my ($this) = shift;
    $_ = shift;

    if (/ControlPanel/) {
	return \&Modules::Zine::ctrl_panel;
    }
    elsif (/isRestricted/) {
	return $config{PhotoForum}{isRestricted};
    }
    elsif (/isService/) {
	return $config{PhotoForum}{isService};
    }
    elsif (/Publish/) {
	return \&Modules::Zine::Zine::publish;
    }
}

sub show {
    my ($this,%opt) = @_;
    if (! $this->approve) {
	my $type = $this->type;
	return $this->error("Sorry, this $type is not available for viewing.");
    }
    my $stat = $this->handler("ZineShow");
    return $stat if ($stat);
    my $out = $this->show_path(%opt);
    $out .= $this->show_article(%opt);

    if ($this->class eq "zine" &&
	($config{PhotoForum}{access} eq "public" || 
	 $share{DB}->level)) {
	my $url = $this->link(zine=>"newarticle",aid=>$this->{article_id});
	$out .= "<table class=\"ZineTools\">
<tr><td align=\"center\">
<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/PhotoForum/new.gif\" height=\"16\" width=\"16\" border=\"0\" alt=\"new photo\" title=\"new photo\">
<br>new photo</a>
</td></tr>
</table>\n";
    }

    $opt{mode} = "article";
    my $aindex = $this->index(%opt);
    $opt{mode} = "comment";
    my $cindex = $this->index(%opt);
    if ($aindex) {
	if ($cindex) { $out .= "<h3>Articles</h3>\n"; }
	$out .= $aindex;
    }
    if ($cindex) {
	if ($aindex) { $out .= "<h3>Comments</h3>\n"; }
	$out .= $cindex;
    }
    return $out;
}

# override the Zine allow_post() rules, so that we can accept article
# postings from the community.

sub allow_post {
    my ($this,$data) = @_;
    if (! $data->{article_id}) {
	if ($data->{type} eq "article") {
	    if ($config{PhotoForum}{access} =~ /public/) {
		return undef;
	    }
	    elsif ($config{PhotoForum}{access} =~ /members/ && $share{DB}->authorize) {
		return undef;
	    }
	    return "<p>You must be logged in to post pictures here.</p>\n";
	}
    }
    return Modules::Zine::Zine::allow_post($this,$data);
}

1;
