package Modules::Query;

#-----------------------------------------------------------------------
#
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------
# Custom Query Tool - execute, report, save, and export arbitrary 
# SQL queries.

use strict;
use ExSite::Config;
use ExSite::Util;
use ExSite::Misc;
use Modules::BaseDCD;

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this,$options) = @_;
    return "";
}

sub ioctl {
    my $this = shift;
    $_ = shift;
    if (/ModuleName/) {
	return "Database Query Tool";
    }
    elsif (/ModuleInfo/) {
	return "The Query Tool allows you to issue direct queries to your
database, and export the resulting data."
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/isRestricted/) {
	return 0;
	return 1;
    }
}

sub ctrl_panel {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    if ($db->authorize < $config{auth}{root_auth_level} - 1) {
	return $this->error("Permission denied");
    }

    if ($this->{input}{query} && $this->{input}{action} ne "save") {
	# execute a query
	$out = $this->query();
    }

    #$out .= &ShowHash($this->{input});

    if ($this->{input}{action} eq "new") {
	$out .= $this->prompt_for_query;
    }
    elsif ($this->{input}{action} eq "delete") {
	$out .= $this->delete_query;
    }
    elsif ($this->{input}{action} eq "save") {
	$out .= $this->save_query;
	$out .= $this->show_saved_queries;
    }
    elsif ($this->{input}{id}) {
	my $q = $db->fetch("query",$this->{input}{id});
	$this->{input}{query} = $q->{query};
	delete $this->{input}{id};
	$this->{saved_query} = 1;
	$out .= $this->query;
    }
    elsif (exists $db->{map}->get_tablemap->{query}) {
	# we appear to support saving queries
	$out .= $this->show_saved_queries;
    }
    else {
	$out .= $this->prompt_for_query;
    }

    return $out;
}

sub show_saved_queries {
    my $this = shift;
    my $db = $share{DB};
    my @queries = $db->fetch_all("query","query_id");
    my @data;
    foreach my $q (@queries) {
	my $qlink = $this->link(action=>undef,id=>$q->{query_id});
	my $dlink = $this->link(id=>$q->{query_id},action=>"delete");
	push @data, [ "<a href=\"$qlink\">$q->{title}</a>",
		      $db->show_data("query","query",$q->{query}), 
		      "<a href=\"$dlink\">delete</a>",
		      ];
    }
    my $newlink = $this->link(id=>undef,action=>"new");
    my $rpt = new ExSite::ReportBuilder(
					title=>"Saved Queries - click on a report name to view report",
					data=>\@data,
					headers=>["Report","Query","Options"],
					tools=>"<a href=\"$newlink\">new query</a>",
					);
    return $rpt->make;
}

sub prompt_for_query {
    my $this = shift;
    my $url = $this->link(action=>undef,id=>undef);
    my $query = $this->{input}{query} || $config{Query}{default};
    my $allow = $config{Query}{allow} || "select";
    return "<h1>Enter your query:</h1>
<p><img src=\"$config{server}{HTMLpath}/_ExSite/images/caution20.gif\"> 
<strong>Warning:</strong>  This tool is for experts only.  Careless use 
could damage the database.</p>
<form method=\"POST\" action=\"$url\">
<p>Query:<br>
<textarea rows=\"4\" cols=\"60\" name=\"query\">
$query
</textarea>
<br>
(Only $allow statements will be accepted.)</p>
<input type=\"submit\">
<input type=\"reset\">
</form>
\n";
}

sub save_query {
    my $this = shift;
    if ($this->{input}{query}) {
	if ($this->{input}{title}) {
	    # unmask quotes etc
	    my $q = &url_unescape($this->{input}{query});
	    $share{DB}->insert("query",{title=>$this->{input}{title},
					query=>$q});
	}
	else {
	    my $url = $this->link();
	    return "<form method=\"POST\" action=\"$url\">
<p>Enter Query Title: <input name=\"title\" size=\"40\"></p>
<input name=\"query\" value=\"$this->{input}{query}\" type=\"hidden\">
<input type=\"submit\" name=\"action\" value=\"save\">
</form>\n";
	}
    }
    else {
	return $this->error("Invalid delete request - no ID");
    }
    return "";
}

sub delete_query {
    my $this = shift;
    if ($this->{input}{id}) {
	my $db = $share{DB};
	$db->trash_key("query",$this->{input}{id});
	&redirect($this->link(action=>undef,id=>undef));
	return undef;
    }
    else {
	return $this->error("Invalid delete request - no ID");
    }
}

sub query {
    my $this = shift;
    my $db = $share{DB};
    my $sql = &url_unescape($this->{input}{query});
    #my $sql = $this->{input}{query};
    my $out;
    
    my $allow = $config{Query}{allow} || "select";
    my $allow_re = $allow;
    $allow_re =~ s/,\s*/|/g;

    if ($sql =~ /^$allow_re/) {
	my $sth = $db->dbh()->prepare($sql);
	if (! $sth) {
	    $out = $this->error("$DBI::errstr; $!");
	}
	elsif (! $sth->execute) {
	    $out = $this->error("$DBI::errstr; $!");
	}
	else {
	    my $data = $sth->fetchall_arrayref;
	    my $names = $sth->{NAME} || [];
	    if (@$data > 0) {
		if ($this->{input}{action} eq "export") {
		    $share{PageBuf}->cancel;
		    print "Content-type: application/vnd.ms-excel\n\n";
		    $this->export($names,$data);
		}
		else {
		    $out .= $this->report($names,$data);
		}
	    }
	    else {
		$out = "<p>Query executed successfully; no data returned.</p>\n";
	    }
	}
	$sth->finish;
    }
    else {
	$out = $this->error("Illegal query.  Only $allow statements are permitted.");
    }
    return $out;
}

sub export {
    my ($this,$names,$data) = @_;
    my $ncol = scalar @{$data->[0]};
    if (@$names == 0) {
	for (1..$ncol) { push @$names, $_; }
    }
    # export the data

    use Spreadsheet::WriteExcel;
    my ($workbook,$worksheet,$format,$col,$row);

    # Create a new Excel workbook
    $workbook = Spreadsheet::WriteExcel->new("-");

    # Add a worksheet
    $worksheet = $workbook->addworksheet();

    #  Add and define a format
    $format = $workbook->addformat(); # Add a format
    $format->set_bold();


    # Write headings
    $col = $row = 0;
    foreach my $h (@$names) {
	$worksheet->write($row, $col, $h, $format);
	$col++;
    }
    $row++;
    
    # write data
    foreach my $d (@$data) {
	for (0..$ncol-1) {
	    $worksheet->write($row, $_, $d->[$_]);
	}
	$row++;
    }

    # done
    $workbook->close();
}


sub report {
    my ($this,$names,$data) = @_;
    my $out;
    if (@$names == 0) {
	my $ncol = scalar @{$data->[0]};
	for (1..$ncol) { push @$names, $_; }
    }
    if (@$names > $config{Query}{max_cols}) {
	my $ncol = scalar @$names;
	$out .= "<p><strong>Warning:</strong> Query exceeded maximum number of displayable
columns.  This HTML report will only show the first $config{Query}{max_cols}
of $ncol columns.  If exported to Excel, all columns will be 
included.</p>\n";
	my @dat = @$names;
	my @subdat = @dat[0..($config{Query}{max_cols}-1)];
	$names = \@subdat;
	foreach my $d (@$data) {
	    my @dat = @$d;
	    my @subdat = @dat[0..($config{Query}{max_cols}-1)];
	    $d = \@subdat;
	}
    }
    if (@$data > $config{Query}{max_rows}) {
	my $nrow = scalar @$data;
	$out .= "<p><strong>Warning:</strong> Query exceeded maximum number of displayable
rows.  This HTML report will only show the first $config{Query}{max_rows}
of $nrow rows.  If exported to Excel, all rows will be included.</p>\n";
	my @dat = @$data;
	my @subdat = @dat[0..($config{Query}{max_rows}-1)];
	$data = \@subdat;
    }
    # sanitize HTML data
    foreach my $d (@$data) {
	foreach my $v (@$d) {
	    if (length $v > 80) {
		$v = substr $v,0,80;
	    }
	    $v =~ s/:cntrl:/?/g;
	}
    }
    my $exporturl = $this->link(action=>undef,id=>undef);
    # mask quotes and other sensitive chars
    my $qdata = &url_escape($this->{input}{query});
    my $savebtn = $this->{saved_query} ? "" : "<input type=\"submit\" name=\"action\" value=\"save\">";
    $qdata =~ s/[\r\n]+/ /g;
    my $exportform = "<form method=\"post\" action=\"$exporturl\"><input type=hidden value=\"$qdata\" name=\"query\">$savebtn<input type=\"submit\" name=\"action\" value=\"export\"></form>\n";
    my $report = new ExSite::ReportBuilder(
					   data=>$data,
					   headers=>$names,
					   title=>$this->{input}{query},
					   tools=>$exportform,
					   );
    $out .= $report->make;
    return $out;
}

1;
