package Modules::TellAFriend;

# Send an email message about the current page

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Mail;
use ExSite::Form;
use ExSite::Captcha;

use Modules::BaseDCD;

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this,$options) = @_;
    my $in = $this->{input};
    if ($in->{from} && $in->{to}) {
	# we have some form input
	return $this->send;
    }
    elsif ($in->{tellafriend}) {
	return $this->mailform();
    }
    else {
	my $url = $this->link(tellafriend=>1);
	return "<a href=\"$url\">$config{TellAFriend}{anchor}</a>";
    }
    return;
}

sub ioctl {
    my ($this) = shift;
    $_ = shift;

    if (/isService/) {
	return 1;
    }
}

sub mailform {
    my ($this) = @_;
    my $formurl = $this->link;
    my $db = $share{DB};
    my ($subject,$message,$from,$to);
    my %data;

    my $url;
    if ($ENV{HTTP_REFERER}) {
	$url = $ENV{HTTP_REFERER};
    }
    else {
	$url = $config{server}{server}.$ENV{REQUEST_URI};
    }

    if ($db) {
	# see if we can find an appropriate from address
	if ($share{Page} && ! $share{Page}->is_publishing) {
	    $db->authorize;
	    my %user = $db->my_user_record();
	    $from = $user{email};
	}
    }
    if (!$subject) {
	# set default subject to the website name
	$subject = "Re: ".$config{site}{name};
	$subject =~ s/\"/\&quot\;/;
    }
    if (!$message) {
	$message = $config{TellAFriend}{message};
    }
    $message .= "\n\n$url\n";
    my ($challenge,$response) = ExSite::Captcha->new->captcha();
    my $crypt = new ExSite::Crypt;
    my $captcha_answer = $crypt->encrypt($response);
    return "<div class=\"TellAFriend\">
$config{TellAFriend}{heading}
<form action=\"$formurl\" method=\"POST\">
<table>
<tr><td>Your&nbsp;email&nbsp;address:</td><td><input type=\"text\" name=\"from\" size=\"40\" value=\"$from\"></td></tr>
<tr><td>Your&nbsp;friend's&nbsp;email&nbsp;address:</td><td><input type=\"text\" name=\"to\" size=\"40\" value=\"$to\"></td></tr>
<tr><td>Subject:</td><td><input type=\"text\" size=\"40\" name=\"subject\" value=\"$subject\"></td></tr>
<tr><td>Personalize your Message:</td><td><textarea name=\"message\" rows=\"6\" cols=\"50\">$message</textarea></td></tr>
<tr><td>
Anti-spam question:</td><td>$challenge<br>
<input type=\"text\" name=\"captcha\">
<input type=\"hidden\" name=\"captcha_answer\" value=\"$captcha_answer\">
</td></tr>
<tr><td></td><td><input type=\"submit\" value=\"Send\"></td></tr>
</table>
</form>
</div>\n";
}

sub send {
    my $this = shift;
    my $out;
    my $in = $this->{input};
    my $addr = $share{DB}->owner_email();
    my @err;
    foreach (split /,/,$addr) {
	if (!/^[\w](\.?[\w\-]+)*@([\w\-]+\.)+[A-Za-z]{2,4}$/) {
	    push @err, "$_: invalid email address";
	}
    }
    if ($in->{from} !~ /^[\w](\.?[\w\-]+)*@([\w\-]+\.)+[A-Za-z]{2,4}$/) {
	push @err, "invalid FROM address";
    }
    if ($in->{to} !~ /^[\w](\.?[\w\-]+)*@([\w\-]+\.)+[A-Za-z]{2,4}$/) {
	push @err, "invalid TO address";
    }
    if (! $in->{message}) {
	push @err, "no message to send";
    }
    if (! $in->{subject}) { 
	push @err, "no subject";
    }
    if (! $in->{captcha}) {
	push @err, "missing captcha";
    }
    else {
	my $crypt = new ExSite::Crypt;
	my $answer = $crypt->decrypt($in->{captcha_answer});
	$in->{captcha} =~ s/^\s+//;
	$in->{captcha} =~ s/\s+$//;
	if (uc($in->{captcha}) ne uc($answer)) {
	    push @err,"Sorry, that was not the correct answer to the anti-spam question.";
	}
    }
    if (@err > 0) {
	$out = "<p class=\"error\">".join("<br>\n",@err)."</p>\n";
	$out .= "<p><a href=\"javascript:history.back()\">&lt;&nbsp;Back</a></p>\n";
    }
    else {
	if (require ExSite::Mail) {
	    my $stat = &SendMail($in->{to},$in->{from},$in->{subject},$in->{message});
	    if ($stat) {
		$out .= "<p class=\"error\">Problem sending message: error code $stat.</p>\n";
	    }
	    else {
		$out .= "<h3>Message sent successfully - thank you!</h3>\n";
	    }
	}
	if ($this->{options}{reply}) {
	    $out .= "<p><a href=\"$this->{options}{reply}\">Back</a></p>\n";
	}
    }
    return $out;
}

1;

