package Modules::Twitter;
#----------------------------------------------------------------------------
#   This plug-in uses HTML adapted from http://twitter.com/badges/html
#   That HTML code is Copyright Twitter.
#
#   If you want to post Twitter updates, then this plug-in requires 
#   the following Perl Modules:
#       Net::Twitter
#       JSON::Any
#       LWP::UserAgent
#   If you merely want to echo a Twitter log, then it requires no
#   special modules.
#-----------------------------------------------------------------------
#
#   Everything else is:
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Section;

# inherit from BaseDCD class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this,$options) = @_;
    my ($out,$out_tweet);
    my %opt = &DecodeString($options);
    my $tw_title = $opt{title} || $config{Twitter}{title} || "Twitter Updates";
    my $tw_user = $opt{user} || $config{Twitter}{user};
    my $tw_count = $opt{count} || $config{Twitter}{count};
    my $tw_admin_level = $config{Twitter}{admin_level} || 3;

    # post tweets

    if ($config{Twitter}{password} && 
	$share{DB}->level() >= $tw_admin_level) {
	# we think we have twitter privs, and we do have local privs
	if (eval "require Net::Twitter") {
	    # the twitter API is available

	    # display a twitter form
	    my $url = $this->link();
	    $out_tweet = "<form action=\"$url\" method=\"post\">
<input type=\"text\" name=\"tweet\" size=\"20\">
<input type=\"submit\" value=\"Tweet!\">
</form>\n";
	    if ($this->{input}{tweet}) {
		# we have form input; post it back to twitter
		my $tw_pass = $config{Twitter}{password};
		if ($tw_pass) {
		    my $tw = new Net::Twitter(user=>$tw_user,
					      password=>$tw_pass);
		    $tw->update($this->{input}{tweet});
		}
	    }
	}
    }
    elsif ($share{Page} && $share{Page}->is_publishing()) {
	# static page; hide the twitter form until we can authenticate the user
	my $url = $this->link();
	$out_tweet = "<p><a href=\"$url\">tweet!</a></p>\n"
    }

    # display recent tweets

    $out .= "
<h2 class=\"twitter-title\">$tw_title</h2>
<ul id=\"twitter_update_list\"></ul>
<script type=\"text/javascript\" src=\"http://twitter.com/javascripts/blogger.js\"></script>
<script text=\"text/javascript\" src=\"http://twitter.com/statuses/user_timeline/$tw_user.json?callback=twitterCallback2&count=$tw_count\"></script>\n";

    return "<!-- Begin Twitter -->
<div id=\"twitter_div\">
$out
$out_tweet
</div>
<!-- End Twitter -->\n";
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleInfo/) {
	return "The Twitter applet displays your recent \"tweets\" on a 
web page.  Among other things, this allows you to update your site from 
a cell phone.  It can also serve as a live update feature that lets you 
blast updates to your user's cell phones or IM programs.  Go to 
Twitter.com for more info.";
    }
    elsif (/ControlPanel/) {
	return \&write;
    }
    return;
}

1;

