package Modules::WebDir;
use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Input;
use Modules::BaseDCD;
use ExSite::ReportBuilder;
use ExSite::Mail;

use vars qw(@ISA);
push(@ISA,"Modules::BaseDCD");

#---------------------------------------------------------------------
sub new {
	my $this = shift;
	my ($obj,$class);
	$obj = {};
	$class = ref($this) || $this;
	bless $obj, $class;
	$class =~ /^(.+?::)+?(.+)$/;
	$obj->{class_name} = $2;
	$obj->{db} = $share{DB};
	$obj->{cgi} = $config{server}{server}.$config{server}{CGIpath};
	$obj->{html} = $config{server}{server}.$config{server}{HTMLpath};
	$obj->read;  # inherits BaseDCD::read()
	$obj->read_conf;
	return $obj;
}
sub ioctl{
	my $this = shift;
	$_ = shift;   # $_ is the ioctl request
	if (/ModuleName/){
		return "Web Directory";
	}elsif (/ControlPanel/){
		return \&control_panel;
	}elsif (/ModuleInfo/){
        	return "The Web Directory module manages and serves a directory of web resources organized in hierarchical topics.";
	}elsif (/Publish/){
		return \&publish;
	}
	return;
}
sub write{
	my ($this,$options) = @_;
	my %opt = &DecodeString($options);
	my $action = $this->{input}{action} || $opt{action};
	$this->section_id($opt{section_id});
	my $out = "";
	# Report a dead link
	if ($action eq "dead_link" && $this->{input}{resource_id}){
		$out = $this->resourceDead($this->{input}{resource_id});
	# Report an inappropriate link
	}elsif ($action eq "inappropriate_link" && $this->{input}{resource_id}){
		$out = $this->resourceInappropriate($this->{input}{resource_id});
	# Report a bad link
	}elsif ($action eq "bad_link" && $this->{input}{resource_id}){
		$out = $this->resourceBad($this->{input}{resource_id});
	# Does the directory search
	}elsif ($action eq "dir_search" && $this->{input}{search_field}){
		$out = $this->directorySearch($this->{input}{search_field});
	# Add a new link
	}elsif ($action eq "new_link"){
		$out = $this->resourceNew();
	# Process new/edit link data
	}elsif ($action eq "post_link"){
		$out = $this->processPost();
		if ($this->admin_mode()){
			$this->republish();
		}
	# Show web directory
	}else{
		$out = $this->webDirectory();
	}
	return $out;
}
#---------------------------------------------------------------------
# Control panel
sub control_panel{
	my $this = shift;
	$this->section_id($this->{input}{section_id});
	my $action = $this->{input}{action};
	
	if ($action eq "republish"){
		$this->republish();
		$action = "";
	}
	
	$this->admin_mode(1);

	my $out;
	# Disable a resource
	if ($this->is_owner() && $action eq "resource_disable" && $this->{input}{resource_id}){
		$out = $this->resourceDisable($this->{input}{resource_id});
	# Reactivate a resource
	}elsif ($this->is_owner() && $action eq "resource_reactivate" && $this->{input}{resource_id}){
		$out = $this->resourceReactivate($this->{input}{resource_id});
	# Edit a resource
	}elsif ($this->is_owner() && $action eq "resource_edit" && $this->{input}{resource_id}){
		$out = $this->resourceEdit($this->{input}{resource_id});
	# Approve a resource
	}elsif ($this->is_owner() && $action eq "resource_approve" && $this->{input}{resource_id}){
		$out = $this->resourceApprove($this->{input}{resource_id});
	# Edit a topic
	}elsif ($this->is_owner() && $action eq "topic_edit" && $this->{input}{topic_id}){
		$out = $this->topicEdit($this->{input}{topic_id});
	# Process post using do
	}elsif ($action eq "process_doform"){
		$out = $this->processDoForm();
	# Delete a topic
	}elsif ($this->is_owner() && $action eq "topic_delete" && $this->{input}{topic_id}){
		$out = $this->topicDelete($this->{input}{topic_id});
		$this->republish();
	# Add a new topic
	}elsif ($action eq "topic_new"){
		$out = $this->topicNew();
		$this->republish();
	# Delete a topic
	}elsif ($action eq "topic_delete" && $this->{input}{topic_id}){
		$out = $this->topicDelete($this->{input}{topic_id});
		$this->republish();
	# Delete a resource
	}elsif ($this->is_owner() && $action eq "resource_delete" && $this->{input}{resource_id}){
		$out = $this->resourceDelete($this->{input}{resource_id});
		$this->republish();
	}elsif (!$this->section_id()){
		$out = $this->set_section_id();
	}else{
		$out = $this->write();
	}
	$this->admin_mode(0);
	return $out;
}
#----------------------------------------------------------------------
sub admin_mode{
	my ($this,$mode) = @_;
	if (defined $mode){
		$this->{admin_mode} = $mode || 0;
	}
	return $this->{admin_mode} || 0;
}
sub Captcha{
	my ($this) = @_;
	if (ref($this->{captcha}) !~ /captcha/i){
		require ExSite::Captcha;
		$this->{captcha} = new ExSite::Captcha(textmode=>$this->{input}{captchamode});
	}
	return $this->{captcha};
}
sub resolve_link{
	my ($this,$in) = @_;
	# $1: path
	# $2: section id
	# $3: action
	# $4: resource id
	$in =~ s/\[\[(.+?)\|(.+?)\|(.+?)\|(.+?)\]\]/$this->link(path=>($1 eq "undef" ? undef : $1),section_id=>($2 eq "undef" ? undef : $2),action=>($3 eq "undef" ? undef : $3),resource_id=>($4 eq "undef" ? undef : $4))/ieg;
	return $in;
}
sub link{
	my ($this,%opt) = @_;
	if ($this->{publishing}){
		my $path = $opt{path} || "undef";
		my $sectionid = $opt{section_id} || "undef";
		my $action = $opt{action} || "undef";
		my $resourceid = $opt{resource_id} || "undef";
		return "[[$path|$sectionid|$action|$resourceid]]";
	}else{
		return $this->SUPER::link(%opt);
	}
	return;
}
sub section_id{
	my ($this,$id) = @_;
	if (defined $id){
		$this->{section_id} = $id;
	}
	if (!$this->{section_id}){
		if (exists($share{Page})){
			$this->{section_id} = $share{Page}->id("section");
		}elsif ($this->{input}{section_id}){
			$this->{section_id} = $this->{input}{section_id};
		}
	}
	return $this->{section_id} || 0;
}
# Modified sub do from Form.pm
# - added republish() calls
sub processDoForm{
	my ($this) = @_;
	my $reply;
	my $out;
	my $redirect = 1;
	my $action;
	if (!$this->{db}->{data}) {
        	if (! $this->{db}->get_data()) {
        		$this->{db}->error("do form: Nothing to do!");
            	}
	}
	if (!$action){
		$action = $this->{db}->get_action();
	}
	$this->{db}->{action} = $action;

	# fetch the destination URL

	if (exists $this->{db}->{data}->{reply}){
        	$reply = $this->{db}->{data}->{reply};
        	delete $this->{db}->{data}->{reply};
        	if ($reply =~ /doform\.cgi/){
        	    	# recursion is perilous
        		$reply = "";
        	}
	}

	# cancel, if requested

	if ($this->{db}->{submit_action} eq "Cancel"){
        	# bail
        	if ($this->{db}->{handler}->{cancel_form}) {
        		my $out = &{$this->{db}->{handler}->{cancel_form}}($this);
        		if ($out){
                		return $out;
        		}
        	}
        	if ($reply){
        		print "Location: $reply\n\n";
        		return;
        	}else{
        		return "<H3>Operation cancelled.</H3>\n";
        	}
	}

	# do everything that touches the database

	$this->{db}->do_db($action);
	if ($this->{db}->errorcheck){
        	# uh-oh!
            	$out = "<H3>Problem:</H3>\n<p>";
            	$out .= $this->{db}->show_diagnostics("error","html");
            	$out .= "</p>\n<P><A HREF=javascript:history.back()>Go back to form.</A></P>\n";
	}elsif ($reply){            	
	    	# republish the whole web directory
	    	$this->republish();
	    	# success; return the user to the original page
            	print "Location: $reply\n\n";
	}elsif ($this->{db}->{output}){
            	# success; return the status report to the user
            	return $this->{output};
	}else{
            	# success; generic confirmation message
            	$out = "<H3>Done!</H3>";
	}
	return $out;
}
#----------------------------------------------------------------------
sub getModuleDiskPath{
	my ($this) = @_;
	return "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/WebDir/top";
}
# Get disck path
sub getDiskPath{
	my ($this) = @_;
	my $sectionid = $this->section_id();	
	my $path = $this->getModuleDiskPath();
	if ($sectionid){
		$path .= "/${sectionid}";
	}
	return $path;
}
# Publish web Directory
sub publish{
	my ($this,$opt) = @_;
	my @sections;
	if ($this->{input}{section_id}){
		push(@sections,$this->{input}{section_id});
	}else{
		my @sites = $this->{db}->my_real_sites();
                foreach my $site(@sites){
                        push(@sections,$site->{section_id});
                }
	}
	
	my $out = "";
	if (scalar(@sections) > 0){
		$out .= $this->mkdir($this->getModuleDiskPath());
		foreach my $sid(@sections){
			$this->section_id($sid);
			# Publish mode begins
			$this->{publishing} = 1;
			my $filepath = $this->getDiskPath;
			# Create file directory
			$out .= $this->mkdir($filepath);
			# Write top level web directory
			$out .= $this->writeToDisk($filepath,"index.html",$this->webDirectory());
			my @topics = $this->fetchTopicBySection();
			foreach my $topic(@topics){
				my $directorypath = $filepath.$this->getPath($topic->{webdir_topic_id});
				my $filename = "index.html";
				# Create file directory based on the directory path
				$out .= $this->mkdir($directorypath);		
				# Readjust links
				my $content = $this->webDirectory($topic->{webdir_topic_id});
				# Write sub leve web directory
				$out .= $this->writeToDisk($directorypath,$filename,$content);
			}
			# Publish mode ends
			$this->{publishing} = 0;
		}
	}else{
		$out = "<p>Nothing to publish!</p>";
	}
	return "" if $opt->{silence};
	return $out;
}

# Background publishing
sub republish{
	my ($this) = @_;
	my $sectionid = $this->section_id();
	require ExSite::Util;
	ExSite::Util::run_publisher("module","WebDir",0,{section_id=>$sectionid,silent=>1});
	return;
}
# Published file check 
# Display the published web directory contents for public only
sub checkPublishFile{
	my ($this,$dirpath) = @_;
	# We are publishing the pages
	return 0 if $this->{publishing};
	# The user is an admin
	return 0 if $this->admin_mode();
	my $dir = $this->getDiskPath.$dirpath;
	my $file = $dir."/index.html";
	# File exists
	return 1 if (-e $file);
	return 0;
}
# Published file check
sub checkPublishTopLevel{
	my ($this) = @_;
	return $this->checkPublishFile();
}
# Fix sublevel file path
sub fixSubLevelPath{
	my ($this,$dirpath) = @_;
	if ($dirpath =~ /\//){
		my @paths = split(/\//,$dirpath);
		$dirpath = $paths[-1];
	}
	return $this->getPath($dirpath);
}
# Published file check
sub checkPublishSubLevel{
	my ($this,$dirpath) = @_;
	$dirpath = $this->fixSubLevelPath($dirpath);
	return $this->checkPublishFile($dirpath);
}
# Get published file contents
sub getFilePublished{
	my ($this,$dirpath) = @_;
	# filename
	my $filename = $this->getDiskPath.$dirpath."/index.html";
	my $out;
	if (open(IFILE,"<$filename")){
		my @lines = <IFILE>;		
		close(IFILE);
		# Output file contents
		$out .= join("",@lines);
		$out = $this->resolve_link($out);
	}else{
		$this->{db}->error("WebDir:: Failed to load $filename : $!");
	        $out .= "<span class=\"error\">Error: $!</span>\n";
	}	
	return $out;
}
# Get published file contents
sub topLevelPublishFile{
	my ($this) = @_;
	return $this->getFilePublished();
}
# Get published file contents
sub subLevelPublishFile{
	my ($this,$dirpath) = @_;
	$dirpath = $this->fixSubLevelPath($dirpath);
	return $this->getFilePublished($dirpath);
}
sub directoryTaintedCheck{
        my ($this,$directory) = @_;
        $directory =~ /^((\/{1}[a-zA-Z0-9_-]+)+)$/;
        my $checked = $1;
        return $checked;
}
sub fileTaintedCheck{
        my ($this,$filename) = @_; 
        $filename =~ /^([a-zA-Z0-9_-]+\.[a-zA-Z0-9_-]+)$/;
        my $checked = $1;
        return $checked;
}

# Create a directory
sub mkdir{
	my ($this,$dirty_directory) = @_;
	# Tainted check
	my $clean_directory = $this->directoryTaintedCheck($dirty_directory);		
        if (!$clean_directory){
                $this->error("mkdir $clean_directory: failed: $dirty_directory - Invalid Directory");
                return "<span class=\"error\">FAILED: $dirty_directory - Invalid Directory</span>\n"
        }
	my $directory = $clean_directory;
	# Directory existed.
	return if (-e $directory);
	my $out = "<p>Create directory $directory ...";
	if (mkdir($directory)){
		$out .= "succeed.";
	}else{
		$this->error("mkdir $directory: failed: $!");
		$out .= "<span class=\"error\">FAILED: $!</span>\n";
	}
	$out .= "</p>";
	return $out;
}
# Write files to the disk
sub writeToDisk{
	# $file: filename
	# $content: file content
	my ($this,$dirty_directory,$dirty_file,$content) = @_;
	# Tainted check
        my $clean_directory = $this->directoryTaintedCheck($dirty_directory);
        my $clean_filename = $this->fileTaintedCheck($dirty_file);
	
        if (!$clean_directory){
                $this->{db}->error("Catalog::publish: $dirty_directory - Invalid Directory");
                return "<span class=\"error\">FAILED: $dirty_directory - Invalid Directory</span>";
        }
        if (!$clean_filename){
                $this->{db}->error("Catalog::publish: $dirty_file - Invalid Filename");
                return "<span class=\"error\">FAILED: $dirty_file - Invalid Filename</span>";
        }
	
	my $file = "${clean_directory}/${clean_filename}";

	my $out = "<p>Writing $file ...";
	if (open(OFILE,">$file")){
		# Write files
		print OFILE $content;
		close(OFILE);
		$out .= "succeed.";
	}else{
		# Error
		$this->{db}->error("WebDir::publish: publish $file failed: $!");
	        $out .= "<span class=\"error\">FAILED: $!</span>\n";
	}
	$out .= "</p>";
	return $out;
}
# Edit a resource
sub resourceEdit{
	# $resource_id: resource id
	my ($this,$resource_id) = @_;
	# Fetch the resource
	my %res = $this->{db}->fetch("webdir_resource",$resource_id);
	my %input;
	# Gather resource data
	foreach my $key(keys(%res)){
		my $hashkey = "resource_".$key;
		$input{$hashkey} = $res{$key};
	}
	# The resource form uses the merge field 'resource_id'(not 'webdir_resource_id') to indicate the primary key of the record.
	$input{resource_id} = $input{resource_webdir_resource_id};
	delete $input{resource_webdir_resource_id};
	# Get the categories that a resource is attached to
	my @category = $this->{db}->fetch_match("webdir_join",{webdir_resource_id=>$resource_id});
	# Get category infor for the drop down list
	$input{category_list_1} = $this->getCategoryList($category[0]{webdir_topic_id});
	$input{category_list_2} = $this->getCategoryList($category[1]{webdir_topic_id});
	$input{category_list_3} = $this->getCategoryList($category[2]{webdir_topic_id});
	my $form = $this->html_resourceForm();
	# Merge source data into the form
	return $this->merge($form,\%input);
}
# Process post data for adding/editing a resource
sub processPost{
	my ($this) = @_;
	# Post query
	my %input = %{$this->{input}};
	# Category info
	my @category;
	my %category;
	# Store resource data
	my %record;
	# Reply url
	my $reply = $input{reply};
	foreach my $key(keys(%input)){
		# All resource related data are store in the resource_.... keys
		next if $key !~ /^resource/i;
		# Category information
		if ($key =~ /^resource_category_(\d+)/i){
			my $cat_id = $1;
			next if !$input{$key};
			# To prevent the system from inserting duplicate links into the directory 
			# when the user selects duplicate categories
			$category{$input{$key}} = 1;
		# Resource information
		}elsif ($key =~ /resource_(.+)/i){
			my $col = $1;
			# Post data use 'resource_id' to store the record id.
			# The resource table uses 'webdir_resource_id' to store the record id.
			# Adjustment needs to be made for record id, but everthing else should remain the same.
			$col = $col eq "id" ? "webdir_resource_id" : $col;
			$record{$col} = $input{$key};
		}			
	}
	my @validation_error;
	# Put the selected categories into an array
	@category = keys(%category);
	if (@category < 1){
		push(@validation_error,"You must select a category for the link.");
	}
	if (!$this->admin_mode()){
		if (!$this->Captcha()->pass()){
			push(@validation_error,"You have entered an incorrect answer for the image.");
		}
	}
	# The user didn't select a category on the form
	if (scalar(@validation_error) > 0){
		# Error message
		$input{custom_message} = "<p>Error: <br>".join("<br>",@validation_error)."</p>";
		# Gather merge field info
		$input{category_list_1} = $this->getCategoryList($input{resource_category_1});
		$input{category_list_2} = $this->getCategoryList($input{resource_category_2});
		$input{category_list_3} = $this->getCategoryList($input{resource_category_3});
		my $form = $this->html_resourceForm();
		# return the user back to the form
		return $this->merge($form,\%input);
	}
	# record id
	my $res_id;
	# insert a new record
	if (!$record{webdir_resource_id}){
		# For admin, the status is set to 'active'
		# For eveyone else, the status is set to 'suggested
		$record{status} = $this->admin_mode() ? "active" : "suggested";
		# Remove the primary key
		delete $record{webdir_resource_id};
		# Insert the record
		$res_id = $this->{db}->insert("webdir_resource",\%record);
		$record{webdir_resource_id} = $res_id;
		# Send email notification to the administrator
		$this->sendNotification(\%record,\@category);
	# update a record
	}else{
		$res_id = $record{webdir_resource_id};
		# Update the record
		$this->{db}->update("webdir_resource",\%record,{webdir_resource_id=>$res_id});	
		# Find all categories that a resource is linked to and delete them
		my @join_records = $this->{db}->fetch_match("webdir_join",{webdir_resource_id=>$res_id});
		foreach my $join(@join_records){
			$this->delete("webdir_join",$join->{webdir_join_id});
		}
	}
	my $error = "<p>
	Error: The system failed to add a new link to the web directory. <br>
	Please contact the administrator(<a href=\"mailto:".$this->adminEmail()."\">".$this->adminEmail()."</a>).<br>
	<input type=\"button\" value=\"&lt; Back\" onclick=\"history.back()\"></p>";
	# Return an error message if update/insert fails
	return $error if !$res_id;
	# Link a resource to topics
	foreach my $cat_id(@category){
		my %join = (
			webdir_resource_id=>$res_id,
			webdir_topic_id=>$cat_id,
		);
		$this->{db}->insert("webdir_join",\%join);
	}
	# Republish web directory
	$this->republish();
	if ($this->admin_mode()){
		$this->redirect($reply);
		return;
	}else{
		return <<END;
<p>
Thank you for submitting your URL.<br>
<a href="$reply">Click here to continue</a>
</p>
END
	}
	return "";
}
# Get adminstrator's email address
sub adminEmail{
	my ($this) = @_;
	my $stat = $this->run_handler("WebDir_adminEmail");
	return $stat if (defined $stat);
	return $this->{db}->owner_email();
}
# Set a resource's status to 'defective-dead'
sub resourceDead{
	# $resource_id: resource id
	my ($this,$resource_id) = @_;
	my $reply = "$config{server}{server}".$this->link(action=>undef,resource_id=>undef);
	return $this->setResourceStatus($resource_id,"defective-dead",$reply);
}
# Set a resource's status to 'defective-bad-topic'
sub resourceBad{
	# $resource_id: resource id
	my ($this,$resource_id) = @_;
	my $reply = "$config{server}{server}".$this->link(action=>undef,resource_id=>undef);
	return $this->setResourceStatus($resource_id,"defective-bad-topic",$reply);
}
# Set a resource's status to 'defective-inappropriate'
sub resourceInappropriate{
	# $resource_id: resource id
	my ($this,$resource_id) = @_;
	my $reply = "$config{server}{server}".$this->link(action=>undef,resource_id=>undef);
	return $this->setResourceStatus($resource_id,"defective-inappropriate",$reply);
}
# Search function
sub directorySearch{
	# $phrase: key phrase entered on the search field
	my ($this,$phrase) = @_;
	my $sectionid = $this->section_id();
	# escape quotes
	$phrase =~ s/"/\\"/g;
	my $sql = <<END;
select 
	a.*, 
	b.webdir_topic_id as topic_id, 
	c.topic as topic_name,
	c.section_id as section_id
from 
	webdir_resource as a 
	left join webdir_join as b on a.webdir_resource_id = b.webdir_resource_id 
	left join webdir_topic as c on b.webdir_topic_id = c.webdir_topic_id 
where 
	(c.section_id = $sectionid or c.section_id = 0 or c.section_id is null or c.section_id = '')
	and (a.anchor like "%$phrase%" or a.description like "%$phrase%") 
	and (a.status like "active" or a.status like "defective%"
END
	$sql .= $this->admin_mode() ? " or status like \"disabled\"" : "";
	$sql .= ") 
	order by a.webdir_resource_id";
	my @results = $this->{db}->custom_query($sql);
	my %result;
	# Reorganize the sql query results so that they are listed by the resource id
	foreach my $res(@results){
		# Extra topic info into a hash
		my %topic = (name=>$res->{topic_name},id=>$res->{topic_id});
		# Remove topic info
		delete $res->{topic_name};
		delete $res->{topic_id};
		my $id = $res->{webdir_resource_id};
		# Index the result by resource id
		$result{$id} = $res if !exists($result{$id});
		# Attach topic info
		push(@{$result{$id}{topics}},\%topic);
	}
	my $noresult = "<p>Your search - <b>$phrase</b> - did not match any webpages.</p>";
	# Gather merge field info
	my %data;
	# Search form
	$data{search} = $this->html_searchForm();
	# Search results
	$data{result} = @results > 0 ? $this->searchResult(\%result) : $noresult;
	# Add new link
	$data{new_link} = $this->html_newResource();
	# Admin reports
	if ($this->admin_mode()){
		$data{report} = $this->adminReport();
	}
	return $this->html_resultListTemplate(\%data);
}
# Send an email notification to the administrator whenever a new link is added to the directory
sub sendNotification{
	# $record: resource record(hash reference)
	# $category: category list(array reference)
	my ($this,$record,$category) = @_;
	# Don't send email notifications when:
	# 1. The link status isn't 'suggested'
	# 2. The record doesn't have an id
	return if (!$record->{webdir_resource_id} || $record->{status} !~ /suggested/i);
	# Email notification has been turned off
	return if !$config{$this->{class_name}}{enable_notification};
	# Gather data for the merge fields
	my %data = (		
		resource_description=>$record->{description},
		resource_url=>$this->html_resourceUrl($record),
		resource_link=>$record->{anchor},
	);
	my @categories;
	if (scalar(@$category) > 0){
		# Find category info
		my $sql = "select * from webdir_topic where (webdir_topic_id = ".join(" or webdir_topic_id = ",@$category).")";
		@categories = $this->{db}->custom_query($sql);
	}
	my @category_names;
	foreach my $cat(@categories){
		push(@category_names,$cat->{topic});
	}
	$data{resource_category} = join(", ",@category_names);
	my $template = $this->html_searchResourceTemplate();
	# Gather data for sending the notification email
	my $to = $this->adminEmail();
	my $from = $this->adminEmail();
	my $subject = "A link has been added into the web directory";
	my $body = $this->merge($template,\%data);
	# Strip html code
	$body =~ s/<(\/)?(.+?)>//g;
	if ($to){
		&SendMail($to,$from,$subject,$body);
	}	
	return;
	
}
# Format search Results
sub formatResult{
	# $result: search result(hash reference)
	my ($this,$result) = @_;
	my @newResult;
	foreach my $key(keys(%$result)){
		# Gather merge field info
		my %res = %{$result->{$key}};		
		my @topics;
		foreach my $topic(@{$res{topics}}){
			push(@topics,"<a href=\"".$this->link(path=>$this->getPath($topic->{id}),action=>undef,search_field=>undef)."\">$topic->{name}</a>");
		}
		$res{resource_category} = join(", ",@topics);
		
		my %res_data = $this->getResourceDisplayData(\%res);
		%res = (%res,%res_data);
		
		my $template = $this->html_searchResourceTemplate();
		# Store formatted results into an array
		push(@newResult,$this->merge($template,\%res));
	}
	return \@newResult;
}
sub getResourceDisplayData{
	my ($this,$res) = @_;
	my %new_res_data = (
		resource_link=>$this->html_resourceLink($res),
		resource_url=>$this->html_resourceUrl($res),
		resource_description=>$res->{description},
		resource_option=>$this->resourceOption($res),
	);
	# Only access level 3 admins get to modify links under section_id 0 topics
	if (!$res->{section_id}){
		if (!$this->{db}->is_admin()){
			$new_res_data{resource_admin_option} = "";
		}
	}else{
		$new_res_data{resource_admin_option} = $this->resourceAdminOptionOther($res);
	}
	return wantarray ? %new_res_data : \%new_res_data;	
}
# Resource user options
sub resourceOption{
	my ($this,$res) = @_;
	my @links = (
		{id=>"dead",label=>"Dead link",url=>"javascript:notify('dead',$res->{webdir_resource_id})",resource_id=>$res->{webdir_resource_id}},
		{id=>"inappropriate",label=>"Inappropriate content",url=>"javascript:notify('inappropriate',$res->{webdir_resource_id})",resource_id=>$res->{webdir_resource_id}},
		{id=>"bad",label=>"Incorrectly categorized",url=>"javascript:notify('bad',$res->{webdir_resource_id})",resource_id=>$res->{webdir_resource_id}},
	);
	return $this->html_resourceOption($res->{webdir_resource_id},\@links);
}
# Search result
sub searchResult{
	# $result: search result(hash reference)
	my ($this,$result) = @_;
	my @result_data = @{$this->formatResult($result)};
	my $out = $this->directoryJavascript();
	$out .= join("",@result_data);
	return $out;
}
sub fetchTopicBySection{
	my ($this) = @_;
	my $sectionid = $this->section_id();
	my $sql = <<END;
select * 
from webdir_topic 
where (section_id = $sectionid or section_id is null or section_id = '' or section_id = 0)
order by topic asc
END
	my @recs = $this->{db}->custom_query($sql);
	return wantarray ? @recs : \@recs;
}
# Fetch topics
sub fetchTopic{
	my ($this,$topic) = @_;
	my $sectionid = $this->section_id();
	my $where = !$topic ? "(parent is null or parent = 0 or parent = '')" : "parent = $topic";
	my $sql = <<END;
select * 
from webdir_topic 
where (section_id = $sectionid or section_id is null or section_id = '' or section_id = 0) and $where 
order by topic asc
END
	my @recs = $this->{db}->custom_query($sql);
	return wantarray ? @recs : \@recs;
}
# Fix topic path
sub topicList{
	# $path: path from the query e.g. '/1/2/3/4/5/6'
	my ($this,$path) = @_;
	my @oldlist = split(/\//,$path);
	my @newlist;
	# remove the first element from array
	for(my $i = 1; $i < @oldlist; $i++){
		push(@newlist,$oldlist[$i]);
	}
	return \@newlist;
}
# Display web directory
sub webDirectory{
	# $dirpath: web directory path(option - mainly used for publishing web directories to the disk)
	my ($this,$dirpath) = @_;
	if ($dirpath){
		# make sure that value of $dirpath begins with a '/'.. e.g. '/12323'
		$dirpath = $dirpath !~ /^\// ? "/$dirpath" : $dirpath;
	}
	# Get path info from the query
	my $path = $dirpath || $this->{input}{path};
	# Fix topic path
	my @topiclist = @{$this->topicList($path)};
	# The selected topic locates on the last element of the array
	my $topicid = $topiclist[-1];
	my $out;
	if (!$topicid){
		# Top level listing
		$out = $this->toplevel($topicid);
	}else{
		# Sub level listing
		$out = $this->sublevel($topicid,$path);
	}
	return $out;
}
sub getPath{
	my ($this,$id) = @_;
	my @paths = $this->pathSearch($id);
	my $out;
	foreach my $path(@paths){
		$out .= "/$path->{webdir_topic_id}";
	}
	return $out;
}
# Search for paths
sub pathSearch{
	# $id: topic id
	my ($this,$id) = @_;
	my %rec = $this->{db}->fetch("webdir_topic",$id);
	# Recursively searching for parent topics of a topic
	if ($rec{parent}){
		return ($this->pathSearch($rec{parent}),\%rec);
	}else{
		return (\%rec);
	}
	return;
}
# Show path info 
sub sublevelPath{
	# $listdata: topic list(array reference)
	my ($this,$listdata) = @_;	
	# Selected category
	my $currentdir = $listdata->[-1];
	# Doing recursive search on parent topics if there is only record in the array, @listdata
	# (This happens when the user clicks on the category link from the search results)
	if (@$listdata == 1){
		my @checklist = $this->pathSearch($currentdir->{webdir_topic_id});
		# Reset the topic list
		$listdata = \@checklist;
	}
	# Remove the selected topic.  Parent topics are used to create clickable link topics.
	# The selected topic isn't required in this case.
	my %current_topic = %{pop(@$listdata)};
	# 'Top' refers to the toplevel(root) directory
	my @pathlist = ("<a href=\"".$this->link(path=>undef)."\">Top</a>");
	# $toppath contains the path info.  It grows after each foreach loop.
	my $toppath;
	foreach my $data(@$listdata){
		# Update path info
		$toppath .= "/$data->{webdir_topic_id}";
		# Create clickable link topics
		my $link = "<a href=\"".$this->link(path=>$toppath)."\">$data->{topic}</a>";
		push(@pathlist,$link);
	}
	# Add the selected topic at the end
	push(@pathlist,$currentdir->{topic});
	my $out = "<p class=\"WebDirPath\">".join(" &gt; ",@pathlist)."</p>";
	$out .= $this->html_directoryTitle({title=>$currentdir->{topic},admin_option=>$this->adminTopicOption(\%current_topic),description=>$currentdir->{description}});
	return $out;
}
# Sub level web directory
sub sublevel{
	# $topicid: selected topic
	# $dirpath: web directory path
	my ($this,$topicid,$dirpath) = @_;
	# Show published files
	if ($this->checkPublishSubLevel($dirpath)){
		return $this->subLevelPublishFile($dirpath);
	}

	my $sql = <<END;
select
	webdir_resource.*,
	webdir_topic.section_id as section_id
from
	webdir_resource
	left join webdir_join on webdir_join.webdir_resource_id = webdir_resource.webdir_resource_id
	left join webdir_topic on webdir_topic.webdir_topic_id = webdir_join.webdir_topic_id
where
	webdir_join.webdir_topic_id = $topicid and 
	(webdir_resource.status like "active" or webdir_resource.status like "defective%"
END
	if ($this->admin_mode()){
		$sql .= " or webdir_resource.status like \"disabled\"";
	}
	$sql .= ") order by webdir_resource.anchor asc";
	my @resources = $this->{db}->custom_query($sql);
	
	
	# Get topic path info
	my @topicList = @{$this->topicList($dirpath)};
	my @topicListData;
	# Fetch topic info
	foreach my $id(@topicList){
		my %rec = $this->{db}->fetch("webdir_topic",$id);
		push(@topicListData,\%rec);
	}
	# Fetch selected topic's child topics
	my @subtopics = $this->fetchTopic($topicid);
	# Gather merge field data
	my %data;
	# Search form
	$data{search} = $this->html_searchForm();
	# Topic path
	$data{path} = $this->sublevelPath(\@topicListData);
	# Child topics
	$data{topic} = $this->html_sublevelDirectory(\@subtopics,$dirpath);
	# Resources
	$data{resource} = $this->html_sublevelResource(\@resources);
	# Suggest a link
	$data{new_link} = $this->html_newResource();
	# Admin reports
	if ($this->admin_mode()){
		$data{report} = $this->adminReport();
	}
	return $this->html_sublevelTemplate(\%data);
}
# Root web directory
sub toplevel{
	my ($this,$topicid) = @_;
	# Show published files
	if ($this->checkPublishTopLevel()){
		return $this->topLevelPublishFile();
	}
	# Fetch all root topics
	my @topics = @{$this->fetchTopic($topicid)};
	my @topicid;
	# Compile a list of root directory id's
	foreach my $topic(@topics){
		push(@topicid,$topic->{webdir_topic_id});
	}
	my @subtopics;
	if (scalar(@topicid) > 0){
		# Fetch all child topics that are linked to the root topics
		my $sql = "select * from webdir_topic where (parent = ".join(" or parent = ",@topicid).")";
		@subtopics = $this->{db}->custom_query($sql);
	}
	my %subtopics;
	# In	dex child topics by parent topoic id
	foreach my $sub(@subtopics){
		$subtopics{$sub->{parent}}{$sub->{topic}} = $sub;
	}
	# Gather merge field data
	my %data;
	# Search form
	$data{search} = $this->html_searchForm();
	# Root web directory
	$data{topic} = $this->html_toplevelDirectory(\@topics,\%subtopics);
	# Heading
	$data{heading} = $config{$this->{class_name}}{toplevel}{heading} || "The web organized by topic into categories.";
	# Admin reports
	if ($this->admin_mode()){
		$data{report} = $this->adminReport();
		$data{new_link} = $this->adminTopLevelNewLink();
	}
	return $this->html_toplevelTemplate(\%data);
}
sub adminTopLevelNewLink{
	my ($this) = @_;
	return "<center><a href=\"".$this->link(action=>"topic_new")."\">Add a new topic</a></center>";
}
# Find the number of descendants on a topic
sub findTopicResource{
	# $topicid: topoic id
	my ($this,$topicid) = @_;	
	my %descendants = %{$this->{db}->find_descendants("webdir_topic",$topicid)};
	my @join_ids = exists($descendants{webdir_join}) ? @{$descendants{webdir_join}} : ();
	my $sql = "select * from webdir_join where webdir_join_id = ".join(" or webdir_join_id = ",@join_ids);
	my @joins = $this->{db}->custom_query($sql);
	my @ids;
	foreach my $join(@joins){
		push(@ids,$join->{webdir_resource_id});
	}
	$sql = "select * from webdir_resource where (webdir_resource_id = ".join(" or webdir_resource_id = ",@ids).")";
	$sql .= " and (status like 'active' or status like 'defective%'";
	$sql .= $this->admin_mode() ? " or status like 'disabled'" : "";
	$sql .= ")";
	my $count = @{$this->{db}->custom_query($sql)};
	return $count;
}
# Reorganize the data for outputting html
sub displayDirectory{
	# $topics: list of topics(array reference
	# $column: number of columns in the table
	my ($this,$topics,$column) = @_;	
	require POSIX;
	# Figure out the number of rows in the table
	my $row = POSIX::ceil(@$topics/$column);
	# Record position in the array, $topics
	my $rec_pos = 0;
	# Organize the data column by column
	my %coldata;
	# Loop throught each column
	for(my $i = 0; $i < $column; $i++){
		# Use modulus to figure out the number of records per column
		my $looprow = (((@$topics % $column) - 1) >= $i) || (@$topics % $column == 0) ? $row : $row - 1;
		# Populate each column with data
		for(my $j = 0; $j < $looprow; $j++){
			# Column number
			my $hashindex = $j + 1;
			# Merge field
			my $dataindex = "row".($i + 1);
			my $topic = $topics->[$rec_pos];
			# Merge field data
			$coldata{$hashindex}{$dataindex} = $topic->{display_data};
			# Increment the index
			$rec_pos++;
		}		
	}
	# Once we have the data formatted column by column, we need to convert
	# the data from a hash to an array.  We merge data by row not by column.
	# Each array represents a row in the table.
	my @rowdata;
	# Numericly sort the hash keys
	foreach my $hindex(sort {$a <=> $b} keys(%coldata)){
		my %hdata = %{$coldata{$hindex}};
		push(@rowdata,\%hdata);
	}
	return \@rowdata;
}
# Input exsite handler
sub handler_input_exsite{
	my ($this,%opt) = @_;
	if ($opt{table} eq "webdir_topic" && $opt{column} eq "parent" && $config{$share{webdir}->{class_name}}{edit}{categorylist}{treemode}){
		my $select = $opt{value};
		my $record = $opt{record};
		my $out = "<select name=\"webdir_topic#$record#parent#key\">";
		$out .= $share{webdir}->getCategoryList($select);
		$out .= "</select>";
		return $out;
	}
	return;
}
# Create a new topic
sub topicNew{
	my ($this,$section_id) = @_;
	my $path = $this->{input}{path};
	my $pid = 0;
	if ($path){
		my @paths = split(/\//,$path);
		$pid = $paths[-1];
	}
	$share{webdir} = $this;
	$this->{db}->handler("input_exsite",\&handler_input_exsite);
	my $reply = "$config{server}{server}".$this->link(action=>undef);
	my %data = (
		parent=>$pid,
		section_id=>$this->section_id(),
	);
	my @hide;
	if (!$this->{db}->is_admin()){
		@hide = ("section_id");
	}
	return $this->{db}->make(table=>"webdir_topic",data=>\%data,hide=>\@hide,extra=>{reply=>$reply});
}
# Edit a topic
sub topicEdit{
	my ($this,$topic_id) = @_;
	$share{webdir} = $this;
	$this->{db}->handler("input_exsite",\&handler_input_exsite);
	my $reply = "$config{server}{server}".$this->link(action=>undef,topic_id=>undef);
	my @hide;
	if (!$this->{db}->is_admin()){
		@hide = ("section_id");
	}
	return $this->{db}->make(table=>"webdir_topic",record=>$topic_id,hide=>\@hide,action=>$this->link(action=>undef,topic_id=>undef),extra=>{action=>"process_doform",reply=>$reply});
}
# Delete a topic
sub topicDelete{
	my ($this,$topic_id) = @_;
	$this->delete("webdir_topic",$topic_id);
	return;
}
# Delete a resource
sub resourceDelete{
	my ($this,$resource_id) = @_;
	$this->delete("webdir_resource",$resource_id);
	return;
}
# Create a new resource
sub resourceNew{
	my ($this) = @_;
	my $reply = $this->link(action=>undef);
	my $form = $this->html_resourceForm();
	# Gather merge field data
	my $newCategoryList = $this->getCategoryList();
	my %data = (
		reply=>"$config{server}{server}".$this->link(action=>undef),
		category_list_1=>$newCategoryList,
		category_list_2=>$newCategoryList,
		category_list_3=>$newCategoryList,
	);
	return $this->merge($form,\%data);
}

# Set a resource's status to 'disabled'
sub resourceDisable{
	# $resource_id: resource record id
	my ($this,$resource_id) = @_;
	return $this->setResourceStatus($resource_id,"disabled");
}
# Set a resource's status to 'active'
sub resourceReactivate{
	# $resource_id: resource record id
	my ($this,$resource_id) = @_;
	return $this->setResourceStatus($resource_id,"active");
}
# Set a resource's status to 'active'
sub resourceApprove{
	# $resource_id: resource record id
	my ($this,$resource_id) = @_;
	return $this->resourceReactivate($resource_id);
}
# Set a resource's status
sub setResourceStatus{
	# $resource_id: resource record id
	# $status: status
	# $reply: reply url
	my ($this,$resource_id,$status,$reply) = @_;
	# No resource is provided
	return if !$resource_id;
	$this->{db}->update("webdir_resource",{status=>$status},{webdir_resource_id=>$resource_id});

	# Don't republish the web directory is the status is set to suggested or defective	
	if ($status !~ /suggested|defect/i){
		$this->republish();
	}
	
	# Redirect url
	$reply = $reply || "$config{server}{server}".$this->link(action=>undef,resource_id=>undef);
	$this->redirect($reply);
	return;
}
# Report option
sub reportOptions{
	# $options: report options(array reference)
	my ($this,$options) = @_;
	# Output html for report options
	my $out;
	foreach my $op (@$options){
		$out .= "<a href=\"$op->{url}\">".$this->getIcon($op->{id},$op->{label})."</a>";
	}
	return $out;
}
# Fetch directory resources by section_id if one is found
sub fetchTopicResource{
	my ($this) = @_;
	my $sectionid = $this->section_id();
	my @recs;
	if ($sectionid){
		my $sql = <<END;
select
	webdir_resource.*
from
	webdir_resource
	left join webdir_join on webdir_join.webdir_resource_id = webdir_resource.webdir_resource_id
	left join webdir_topic on webdir_topic.webdir_topic_id = webdir_join.webdir_topic_id
where
	webdir_topic.section_id = $sectionid 	
END
		if ($this->admin_mode()){
			if ($this->{db}->is_admin()){
				$sql .= " or webdir_topic.section_id = 0 or webdir_topic.section_id = '' or webdir_topic.section_id is null";
			}
		}
		@recs = $this->{db}->custom_query($sql);
	}else{
		@recs = $this->{db}->fetch_all("webdir_resource");
	}
	return wantarray ? @recs : \@recs;
}
# Admin Reports
sub adminReport{
	my ($this) = @_;
	if (!$this->is_owner()){
		return;
	}
	# Fetch all resources
	my @resources = $this->fetchTopicResource();
	# Resource report headers
	my @link_header = ("Options","URL","Anchor Text","Description");
	# Defective resources
	my @defect;
	# Suggested resources
	my @suggest;
	# Group resources by status
	foreach my $resource(@resources){
		my $url = $this->{db}->show_data("webdir_resource","url",$resource->{url},$resource->{webdir_resource_id});
		# Defective resources
		if ($resource->{status} =~ /^defect/i){
			# report options	
			my $ops = $this->resourceAdminOptionDefect($resource);
			my $status = $resource->{status};
			$status =~ s/defective-//gi;
			push(@defect,[$ops,$url,$resource->{anchor},$resource->{description},$status]);
		# Suggested resources
		}elsif ($resource->{status} =~ /^suggest/i){
			# report options
			my $ops = $this->resourceAdminOptionSuggest($resource);
			push(@suggest,[$ops,$url,$resource->{anchor},$resource->{description}]);		
		}
	}
	# Create reports
	my $suggest = "<p>".$this->reportbuilder({title=>"Suggested Resources",headers=>\@link_header,data=>\@suggest})."</p>";
	push(@link_header,"Status");
	my $defect = "<p>".$this->reportbuilder({title=>"Defective Resources",headers=>\@link_header,data=>\@defect})."</p>";	
	my $out = $this->javascript();
	$out .= $suggest.$defect;
	return $out;	
}
# Admin topic options
sub adminTopicOption{
	my ($this,$topic) = @_;
	# Report options
	my @ops = (
		{id=>"new",url=>$this->link(action=>"topic_new",resource_id=>undef),label=>"New Topic"},
		{id=>"edit",url=>$this->link(action=>"topic_edit",topic_id=>$topic->{webdir_topic_id},resource_id=>undef),label=>"Edit '$topic->{topic}'"},
	);
	# Can this directory be deleted?
	my $removable = $this->is_webdir_removable($topic->{webdir_topic_id});
	if ($removable){
		push(@ops,{id=>"delete",url=>"javascript:deleteTopic($topic->{webdir_topic_id})",label=>"Delete '$topic->{topic}'"});
	}
	# Only access level 3 users get to update global directories
	if (!$this->{db}->is_admin() && !$topic->{section_id}){
		@ops = ();
	}
	return $this->reportOptions(\@ops);
}
# Find child topics and resources
sub findChildTopicResource{
	# $topic_id: topic id
	my ($this,$topic_id) = @_;
	# Find all descendants of a tpoic
	my %descendants = %{$this->{db}->find_descendants("webdir_topic",$topic_id)};
	# Child topics
	my $topic = scalar(@{$descendants{webdir_topic}}) == 1 ?  0 : scalar(@{$descendants{webdir_topic}}) - 1;
	# Child resources
	my $resource = exists($descendants{webdir_join}) ? scalar(@{$descendants{webdir_join}}) : 0;
	return ($topic,$resource);
}
# Resource admin options(active/disabled resources)
sub resourceAdminOptionOther{
	# $resource: hash reference
	my ($this,$resource) = @_;
	my @ops = (
		{id=>"edit",url=>$this->link(action=>"resource_edit",resource_id=>$resource->{webdir_resource_id}),label=>"Edit '$resource->{anchor}'"},
		{id=>"delete",url=>"javascript:deleteResource($resource->{webdir_resource_id})",label=>"Delete '$resource->{anchor}'"},
	);
	if ($resource->{status} =~ /^disabled/i){
		push(@ops,{id=>"reactivate",url=>$this->link(action=>"resource_reactivate",resource_id=>$resource->{webdir_resource_id}),label=>"Reactivate '$resource->{anchor}'"});
	}else{
		push(@ops,{id=>"disable",url=>$this->link(action=>"resource_disable",resource_id=>$resource->{webdir_resource_id}),label=>"Disable '$resource->{anchor}'"});
	}
	return $this->reportOptions(\@ops);
}
# Resource admin options(defective resources)
sub resourceAdminOptionDefect{
	# $resource: hash reference
	my ($this,$resource) = @_;
	my @ops = (
		{id=>"edit",url=>$this->link(action=>"resource_edit",resource_id=>$resource->{webdir_resource_id}),label=>"Edit '$resource->{anchor}'"},
		{id=>"disable",url=>$this->link(action=>"resource_disable",resource_id=>$resource->{webdir_resource_id}),label=>"Disable '$resource->{anchor}'"},
		{id=>"reactivate",url=>$this->link(action=>"resource_reactivate",resource_id=>$resource->{webdir_resource_id}),label=>"Reactivate '$resource->{anchor}'"},
		{id=>"delete",url=>"javascript:deleteResource($resource->{webdir_resource_id})",label=>"Delete '$resource->{anchor}'"},
	);
	return $this->reportOptions(\@ops);
}
# Resource admin options(suggested resources)
sub resourceAdminOptionSuggest{
	# $resource: hash reference
	my ($this,$resource) = @_;
	my @ops = (
		{id=>"edit",url=>$this->link(action=>"resource_edit",resource_id=>$resource->{webdir_resource_id}),label=>"Edit '$resource->{anchor}'"},
		{id=>"approve",url=>$this->link(action=>"resource_approve",resource_id=>$resource->{webdir_resource_id}),label=>"Approve '$resource->{anchor}'"},
		{id=>"delete",url=>"javascript:deleteResource($resource->{webdir_resource_id})",label=>"Delete '$resource->{anchor}'"},
	);
	return $this->reportOptions(\@ops);
}
# Get icons
sub getIcon{
	my ($this,$id,$label) = @_;
	my %icon = (
		edit=>{label=>"Edit",image=>"edit12.gif"},
		approve=>{label=>"Approve",image=>"approve12.gif"},
		disable=>{label=>"Disable",image=>"disable12.gif"},
		reactivate=>{label=>"Reactivate",image=>"approve12.gif"},
		delete=>{label=>"Delete",image=>"rm12.gif"},
		new=>{label=>"New",image=>"new12.gif"},
	);
	$label = $label || $icon{$id}{label};
	my %data = (
		image_url=>"$this->{html}/_Modules/$this->{class_name}/$icon{$id}{image}",
		image_label=>$label,
		image_border=>0,
		image_hspace=>2,
		image_align=>"middle",
	);
	return $this->html_icon(\%data);
}
#------------------------------------------------------------------------------------------------
# Icon template
sub html_icon{
	my ($this,$data) = @_;
	my $template = "<img src=\"<!--[[image_url]]-->\" border=\"<!--[[image_border]]-->\" hspace=\"<!--[[image_hspace]]-->\" ";
	$template .= "align=\"<!--[[image_align]]-->\" title=\"<!--[[image_label]]-->\" alt=\"<!--[[image_label]]-->\">";
	return $this->merge($template,$data);
}
# Resource link
sub html_resourceLink{
	my ($this,$rec) = @_;
	my $url = $this->html_resourceUrl($rec);
	my $out = "<a href=\"$url\"";
	if ($config{$this->{class_name}}{show_link_openwindow}){
		$out .= " target=\"_blank\"";
	}
	$out .= ">$rec->{anchor}</a>";
	return $out;
}
# Resource url
sub html_resourceUrl{
	my ($this,$rec) = @_;
	my $out = $rec->{url};
	$out = $out =~ /^http:\/\// ? $out : "http://$out";
	return $out;
}
# Template for showing a single resource
sub html_sublevelResourceTemplate{
	my ($this) = @_;
	return $this->html_sublevelResourceTemplateAdmin() if $this->admin_mode();
	return $this->html_sublevelResourceTemplatePublic();
}
# Template for showing a single resource - admin
sub html_sublevelResourceTemplateAdmin{
	my ($this) = @_;
	my $out = "
<span class=\"WebDirName\"><!--[[resource_link]]--></span> - <span class=\"WebDirUrl\"><!--[[resource_url]]--></span> <!--[[resource_admin_option]]--><br>
<span class=\"WebDirDescription\"><!--[[resource_description]]--></span><br>
<span class=\"WebDirNotify\"><!--[[resource_option]]--></span>
	";
	return $out;
}
# Template for showing a single resource - public
sub html_sublevelResourceTemplatePublic{
	my ($this) = @_;
	my $out = "
<span class=\"WebDirName\"><!--[[resource_link]]--></span> - <span class=\"WebDirUrl\"><!--[[resource_url]]--></span><br>
<span class=\"WebDirDescription\"><!--[[resource_description]]--></span><br>
<span class=\"WebDirNotify\"><!--[[resource_option]]--></span>
	";
	return $out;
}
# Template for showing a single resource - public
sub html_searchResourceTemplatePublic{
	my ($this) = @_;
	my $out = "
<p>
<span class=\"WebDirName\"><!--[[resource_link]]--></span><br>
Category: <!--[[resource_category]]--><br>
<span class=\"WebDirDescription\"><!--[[resource_description]]--></span><br>
<span class=\"WebDirUrl\"><!--[[resource_url]]--></span><br>
<span class=\"WebDirNotify\"><!--[[resource_option]]--></span>
</p>
	";
	return $out;	
}
# Template for showing a single resource - admin
sub html_searchResourceTemplateAdmin{
	my ($this) = @_;
	my $out = "
<p>
<span class=\"WebDirName\"><!--[[resource_link]]--></span> <!--[[resource_admin_option]]--> <br>
Category: <!--[[resource_category]]--><br>
<span class=\"WebDirDescription\"><!--[[resource_description]]--></span><br>
<span class=\"WebDirUrl\"><!--[[resource_url]]--></span><br>
<span class=\"WebDirNotify\"><!--[[resource_option]]--></span>
</p>
	";
	return $out;	
}
# Template for showing a single resource
sub html_searchResourceTemplate{
	my ($this) = @_;
	return $this->html_searchResourceTemplateAdmin() if $this->admin_mode();
	return $this->html_searchResourceTemplatePublic();
}
# Template for showing a list of results
sub html_resultListTemplate{
	my ($this,$data) = @_;
	my $template = $this->admin_mode() ? $this->html_resultListTemplateAdmin() : $this->html_resultListTemplatePublic();
	return $this->merge($template,$data);
}
# 'Suggest a link' link
sub html_newResource{
	my ($this) = @_;
	my $out = "<p><center><a href=\"".$this->link(action=>"new_link")."\">Missing link? Suggest a link.</a></center></p>";
	return $out;
}
sub html_directoryTitleAdmin{
	my ($this) = @_;
	return <<END;
<h1><!--[[title]]--> <!--[[admin_option]]--></h1>
<p><!--[[description]]--></p>
END
}
sub html_directoryTitlePublic{
	my ($this) = @_;
	return <<END;
<h1><!--[[title]]--></h1>
<p><!--[[description]]--></p>
END
}
# Directory title
sub html_directoryTitle{
	my ($this,$data) = @_;
	my $template = $this->admin_mode() ? $this->html_directoryTitleAdmin() : $this->html_directoryTitlePublic();
	return $this->merge($template,$data);
}
# List resources
sub html_sublevelResource{
	# $resources: list of resources(array reference)
	my ($this,$resources) = @_;
	my $out = "\n<h3>Web pages</h3>";
	# No resources found
	return $out."<p>No web pages found.</p>" if @$resources < 1;
	# Resource options(dead link, inappropriate link....etc)
	$out .= $this->directoryJavascript();
	foreach my $res(@$resources){
		# Gather data for merge fields
		my %data = $this->getResourceDisplayData($res);
		my $template = $this->html_sublevelResourceTemplate();
		# Populate the output html
		$out .= "\n<p>";
		$out .= $this->merge($template,\%data);	
		$out .= "\n</p>";
	}
	return $out;
}
# Resource options
sub html_resourceOption{
	my ($this,$resourceid,$links) = @_;
	my $out = "\n<form>";
	$out .= "<select class=\"WebDirNotifySelect\" id=\"resource_$resourceid\" onChange=\"notify(this.id,$resourceid)\">";
	$out .= "<option value=\"\">== Bad Link ? ==</option>";
	foreach my $link (@$links){
		$out .= "<option value=\"$link->{id}\">$link->{label}</option>";
	}
	$out .= "</select>";
	$out .= "\n</form>";
	return $out;
}
# Javascript for resource options
sub directoryJavascript{
	my ($this) = @_;
	my $out = "
<script language=\"javascript\">
<!--
function notify(element_id,id){
	var alertmessage = 'Notify the administrator that this link ';
	var option = document.getElementById(element_id).value;
	switch(option){
		case 'dead':
			alertmessage += ' is dead.';
			break;
		case 'inappropriate':
			alertmessage += ' contains inappropriate contents.';
			break;
		case 'bad':
			alertmessage += ' is incorrectly categorized.';
			break;
		default:
			return;
	}
	if (confirm(alertmessage)){
		location.href = '".$this->link(action=>undef,resource_id=>undef)."&action=' + option + '_link&resource_id=' + id;
	}
}
//-->
</script>
	";
	return $out;
}
# Template for root web directory
sub html_toplevelTemplate{
	my ($this,$data,$opt) = @_;
	my $template = $this->admin_mode() ? $this->html_toplevelTemplateAdmin() : $this->html_toplevelTemplatePublic();
	return $this->merge($template,$data);
}

# Template for root web directory
sub html_sublevelTemplate{
	my ($this,$data,$opt) = @_;
	my $template = $this->admin_mode() ? $this->html_sublevelTemplateAdmin() : $this->html_sublevelTemplatePublic();
	return $this->merge($template,$data);
}
# Web directory search form
sub html_searchForm{
	my ($this) = @_;
	my $out = "
<center>
<form method=\"post\" action=\"".$this->link(action=>undef,path=>undef)."\">
<input type=\"hidden\" name=\"action\" value=\"dir_search\">
<table>
<tr><td><input type=\"text\" size=\"30\" name=\"search_field\" value=\"\"></td><td><input type=\"submit\" value=\"Search\"></td></tr>
</table>
</form>
</center>
	";
	return $out;
}
# Template for table rows
sub html_sublevelRowTemplate{
	my ($this,$cols) = @_;
	$cols = !$cols ? 3 : $cols;
	my $out;
	for(my $i = 0; $i < $cols; $i++){
		my $merge_key = "row".($i + 1);
		$out .= "<td valign=\"top\"><!--[[$merge_key]]--></td>";
	}
	return $out;
}
# Root webdirectory topic link - public
sub html_directoryCellTitleLinkPublic{
	my ($this) = @_;
	return "<a href=\"<!--[[url]]-->\"><!--[[topic]]--></a>";
}
# Root webdirectory topic link - admin
sub html_directoryCellTitleLinkAdmin{
	my ($this) = @_;
	return "<a href=\"<!--[[url]]-->\"><!--[[topic]]--></a> <!--[[admin_option]]-->";
}
# Root webdirectory topic link
sub html_directoryCellTitleLink{
	my ($this,$data) = @_;
	my $template = $this->admin_mode() ? $this->html_directoryCellTitleLinkAdmin() : $this->html_directoryCellTitleLinkPublic();
	return $this->merge($template,$data);
}
# Create clickable link topic
sub html_sublevelCell{
	my ($this,$topic,$dirpath) = @_;
	my $path = "$dirpath/$topic->{webdir_topic_id}";
	my $out = $this->html_directoryCellTitleLink({url=>$this->link(path=>"/$topic->{webdir_topic_id}"),topic=>$topic->{topic},admin_option=>$this->adminTopicOption($topic)});
	return $out if !$config{$this->{class_name}}{show_resource_count};
	my $resources = $this->findTopicResource($topic->{webdir_topic_id});
	$out .= " ($resources)";
	return $out;
}
# Show the child topics
sub html_sublevelDirectory{
	# $topics: list of topics(array reference)
	my ($this,$topics,$dirpath) = @_;
	# No topics found
	return if @$topics < 1;
	# Create clickable link for every topics
	for(my $i = 0; $i < @$topics; $i++){
		$topics->[$i]{display_data} = $this->html_sublevelCell($topics->[$i],$dirpath);
	}
	# The number of columns the table is going to show
	my $column = $config{$this->{class_name}}{sublevel}{display_column} || 3;
	# Convert topic data into merge field data
	my @rowdata = @{$this->displayDirectory($topics,$column)};
	# Get template for table rows
	my $rowtemplate = $this->html_sublevelRowTemplate($column);
	# Output the page
	my $out;
	$out .= "\n<h3>Categories</h3>";
	$out .= "\n<table width=\"100%\" class=\"WebDirCategories\">";
	foreach my $rowdat(@rowdata){
		$out .= "\n<tr>";
		$out .= $this->merge($rowtemplate,$rowdat);
		$out .= "</tr>";
	}
	$out .= "\n</table>";
	return $out;
}
# Show the root topics
sub html_toplevelDirectory{
	# $topics: list of root topics(array reference)
	# $subtopics: list of child topics(hash reference)
	my ($this,$topics,$subtopics) = @_;
	# No root topics found
	return "<p>No categories found</p>" if @$topics < 1;
	# The number of child topics to be shown under a root topic
	my $subtopicPerCell = $config{$this->{class_name}}{toplevel}{childtopic_display} || 2;
	# Create clickable link topics
	for(my $i = 0; $i < @$topics; $i++){
		$topics->[$i]{display_data} = $this->html_toplevelCell($subtopicPerCell,$topics->[$i],$subtopics->{$topics->[$i]{webdir_topic_id}});
	}
	# The number of columns the table is going to show
	my $column = $config{$this->{class_name}}{toplevel}{display_column} || 3;
	# Convert topic data into merge field data
	my @rowdata = @{$this->displayDirectory($topics,$column)};
	# Get template for table rows
	my $rowtemplate = $this->html_toplevelRowTemplate($column);
	# Output the page
	my $out = "<table class=\"WebDirectory\" cellspacing=\"10\" cellpadding=\"5\">";	
	foreach my $rowdat(@rowdata){
		$out .= "\n<tr>";
		$out .= $this->merge($rowtemplate,$rowdat);
		$out .= "</tr>";
	}
	$out .= "</table>";
	return $out;
}
# Display for each cell in the root web directory page
sub html_toplevelCell{
	# $limit: number of child topics to show
	# $topic: current topic(hash reference)
	# $subtopics: list of child topics(hash reference)
	my ($this,$limit,$topic,$subtopics) = @_;
	# Root topic clickable link
	my $topiclink = $this->html_directoryCellTitleLink({url=>$this->link(path=>"/$topic->{webdir_topic_id}"),topic=>$topic->{topic},admin_option=>$this->adminTopicOption($topic)});
	# Convert child topics from hashes to arrays
	my @stopics = sort keys(%$subtopics);
	# Store child topic html code in an array
	my @sub_topics;
	for(my $i = 0; $i < $limit; $i++){
		# Exit the loop when the number of child records is less than the $limit
		last if (@stopics < ($i + 1));
		# The child topic that we are walking on
		my $currtopic = $subtopics->{$stopics[$i]};
		# Create clickable link
		my $subtopiclink = "<a href=\"".$this->link(path=>"/$topic->{webdir_topic_id}/$currtopic->{webdir_topic_id}")."\">$currtopic->{topic}</a>";
		push(@sub_topics,$subtopiclink);
	}
	# Output the page
	my $out = "<b>$topiclink</b>";
	$out .= "<br>" if keys(%$subtopics) > 0;
	if (@sub_topics > 0){
		$out .= join(", ",@sub_topics)."...";
	}
	return $out;
}
# Template for table rows(root web directory)
sub html_toplevelRowTemplate{
	my ($this,$cols) = @_;
	$cols = !$cols ? 3 : $cols;
	my $out;
	for(my $i = 0; $i < $cols; $i++){
		my $merge_key = "row".($i + 1);
		$out .= "<td valign=\"top\"><!--[[$merge_key]]--></td>";
	}
	return $out;
}
#----------------------------------------------------------------------
# Admin resource style
sub html_adminResourceStyle{
	my ($this) = @_;
	my $out = "
<style type=\"text/css\">
.WebDirUrl{
	color: #6f6f6f;
	font-size: 12px;
}
.WebDirDescription{
	font-size: 12px;
}
.WebDirNotifySelect{
	font-size: 8px;
	color: #6f6f6f;
}
</style>
";
	return $out;
}
# Template for showing a list of results - public
sub html_resultListTemplatePublic{
	my ($this) = @_;
	my $out = "
<div class=\"WebDir\">
<!--[[search]]-->
<h3>Directory</h3>
<table>
<tr><td><!--[[result]]--></td></tr>
</table>
<!--[[new_link]]-->
</div>
	";
	return $out;
}
# Template for showing a list of results - admin
sub html_resultListTemplateAdmin{
	my ($this) = @_;
	my $out = $this->html_adminResourceStyle();
	$out .= "
<div class=\"WebDir\">
<!--[[report]]-->
<hr>
<!--[[search]]-->
<h3>Directory</h3>
<table>
<tr><td><!--[[result]]--></td></tr>
</table>
<!--[[new_link]]-->
</div>
	";
	return $out;
}
# Template for root web directory - admin
sub html_toplevelTemplateAdmin{
	my ($this) = @_;
	return <<END;
<div class="WebDir">
<!--[[report]]-->
<hr>
<!--[[search]]-->
<h3 align="center"><!--[[heading]]--></h3>
<hr>
<center>
<!--[[topic]]-->
</center>
<hr>
<!--[[new_link]]-->
</div>
END
}
# Template for root web directory - public
sub html_toplevelTemplatePublic{
	my ($this) = @_;
	return <<END;
<div class="WebDir">
<!--[[search]]-->
<h3 align="center"><!--[[heading]]--></h3>
<hr>
<center>
<!--[[topic]]-->
</center>
<hr>
</div>
END
}
# Sublevel web directory page - public
sub html_sublevelTemplateAdmin{
	my ($this) = @_;
	my $template = $this->html_adminResourceStyle();
	$template .= "
<style type=\"text/css\">
.WebDirUrl{
	color: #6f6f6f;
	font-size: 12px;
}
.WebDirDescription{
	font-size: 12px;
}
.WebDirNotifySelect{
	font-size: 8px;
	color: #6f6f6f;
}
</style>
<div class=\"WebDir\">
<!--[[report]]-->
<hr>
<!--[[search]]-->
<!--[[path]]-->
<!--[[topic]]-->
<!--[[resource]]-->
<!--[[new_link]]-->
</div>
	";
	return $template;
}
# Sublevel web directory page - public
sub html_sublevelTemplatePublic{
	my ($this) = @_;
	my $template = "
<div class=\"WebDir\">
<!--[[search]]-->
<!--[[path]]-->
<!--[[topic]]-->
<!--[[resource]]-->
<!--[[new_link]]-->
</div>
	";
	return $template;
}
# Form for adding/editing a resource
sub html_resourceForm{
	my ($this) = @_;
	my $captcha = $this->Captcha()->make();
	if ($this->admin_mode()){
		$captcha = "";
	}
	my $out = "
<div class=\"WebDir\">
<h2>Suggest a link</h2>
<!--[[custom_message]]-->
<form method=\"post\" action=\"".$this->link(action=>undef)."\">
<input type=\"hidden\" name=\"reply\" value=\"<!--[[reply]]-->\">
<input type=\"hidden\" name=\"action\" value=\"post_link\">
<input type=\"hidden\" name=\"resource_id\" value=\"<!--[[resource_id]]-->\">
<p><span class=\"label\">Category 1</span><br>
<select name=\"resource_category_1\">
<!--[[category_list_1]]-->
</select>
</p>
<p><span class=\"label\">Category 2</span><br>
<select name=\"resource_category_2\">
<!--[[category_list_2]]-->
</select>
</p>
<p><span class=\"label\">Category 3</span><br>
<select name=\"resource_category_3\">
<!--[[category_list_3]]-->
</select>
</p>
<p><span class=\"label\">URL</span><br><input type=\"text\" size=\"80\" name=\"resource_url\" value=\"<!--[[resource_url]]-->\"></p>
<p><span class=\"label\">Anchor text</span><br><input type=\"text\" size=\"80\" name=\"resource_anchor\" value=\"<!--[[resource_anchor]]-->\"></p>
<p><span class=\"label\">Description</span><br><textarea wrap=\"virtual\" rows=\"4\" cols=\"80\" name=\"resource_description\"><!--[[resource_description]]--></textarea></p>
<p>
$captcha
</p>
<p><input type=\"submit\" value=\"Submit\">&nbsp;&nbsp;<input type=\"reset\" value=\"Reset\"></p>
</form>
</div>
	";
	return $out;	
}
#--------------------------------------------------------------------
# Build data tree
sub buildDataTree{
	# $rec: record
	# $data: a list containing records indexed by their parent record
	# $primary_key: name of the primary key column
	my ($this,$rec,$data,$primary_key) = @_;
	# Null record
	return undef if !exists($rec->{$primary_key});
	my @data;
	# Look for descendant records
	if (exists($data->{$rec->{$primary_key}})){
		@data = @{$data->{$rec->{$primary_key}}};
	}
	# Build sub tree
	foreach my $dat(@data){
		push(@{$rec->{child_record}},$this->buildDataTree($dat,$data,$primary_key));
	}
	return $rec;
}
# Get all categories in the drop down list format - tree
sub html_getCategoryList_tree{
	# $label_col: label column
	# $data: list of data
	# $primary_key: name of the primary key
	# $foreign_key: name of the parent key
	# $select: selected value 
	my ($this,$data,$label_col,$primary_key,$foreign_key,$select) = @_;
	my %data_index;
	# Index records by its foreign key
	foreach my $dat(@$data){
		my $index = $dat->{$foreign_key} || 0;
		push(@{$data_index{$index}},$dat);
	}
	my %result = %{$this->buildDataTree({$primary_key=>0,$label_col=>"== select =="},\%data_index,$primary_key)};
	return $this->html_displayTreeStyleList(\%result,$primary_key,$label_col,$select);
}
sub html_displayTreeStyleList{
	# $data: data to be shown(hash reference)
	# $primary_key: name of the primary key column
	# $label_col: name of the label column
	# $select: selected value
	# $sep: a string used to show different tree levels
	my ($this,$data,$primary_key,$label_col,$select,$sep) = @_;
	my $out;
	$out .= "<option value=\"$data->{$primary_key}\"";
	$out .= $select == $data->{$primary_key} ? " selected" : "";
	$out .= ">$sep$data->{$label_col}</option>";
	# Display descendant records recursively
	foreach my $dat(@{$data->{child_record}}){
		my $separator .= $sep.$config{$this->{class_name}}{edit}{categorylist}{treemode_separator};
		$out .= $this->html_displayTreeStyleList($dat,$primary_key,$label_col,$select,$separator);
	}
	return $out;
}
# Get all categories in the drop down list format - list
sub html_getCategoryList_list{
	# $data: list of records
	# $select: selected value
	my ($this,$data,$select) = @_;	
	my $out = "<option value=\"\">== Select ==</option>";
	# Aphabetically sorted list
	foreach my $cat (@$data){
		my $id = $cat->{webdir_topic_id};
		$out .= "<option value=\"$id\"";
		$out .= $select == $id ? " selected" : "";
		$out .= ">$cat->{topic}</option>";
	}
	return $out;
}
# Get all categories in the drop down list format
sub getCategoryList{
	# $select: selected value
	my ($this,$select) = @_;
	my @categories = $this->fetchTopicBySection();
	# Check and see if the treemode is on
	if ($config{$this->{class_name}}{edit}{categorylist}{treemode}){
		return $this->html_getCategoryList_tree(\@categories,"topic","webdir_topic_id","parent",$select);
	}else{		
		return $this->html_getCategoryList_list(\@categories,$select);
	}
	return;
}
#----------------------------------------------------------------------

# Delete confirmation popup
sub javascript{
	my ($this) = @_;
	my $url_del_resource = $this->link(action=>"resource_delete");
	my $url_del_topic = $this->link(action=>"topic_delete");
	return <<END;
<script language="javascript">
<!--
	function deleteResource(id){
		if (confirm('Are you sure want to delete this resource?')){
			location.href = '${url_del_resource}&resource_id=' + id;
		}
	}
	function deleteTopic(id){
		if (confirm('Are you sure you want to delete this topic?')){
			location.href = '${url_del_topic}&topic_id=' + id;
		}
	}
//-->
</script>
END
}
# Build reports using ExSite::ReportBuilder
sub reportbuilder{
	my ($this,$opt) = @_;
	require ExSite::ReportBuilder;
	$this->{rb} = new ExSite::ReportBuilder() if !$this->{rb};
	my $rb = $this->{rb};
	$rb->init(
		title=>$opt->{title},
		data=>$opt->{data},
		tools=>$opt->{tools},
		headers=>$opt->{headers},
		foot=>$opt->{foot},
		width=>$opt->{width},
	);
 	return $rb->make();
}
# Page redirection
sub redirect{
        my ($this,$reply) = @_;
	$reply = $reply || $this->link(action=>undef);
	if ($reply !~ /^http:\/\//){
		$reply = "$config{server}{server}$reply";
	}
        my $p = $share{Page} || $share{PageBuf};
        if ($p){
                $p->redirect($reply);
        }else{
                print "location: $reply\n\n";
        }
        return;
}
# Data merge
sub merge{
        my ($this,$view,$data) = @_;
	return if !$view || !$data;
        $view =~ s/\<\!\-\-\[\[((\w+(\_| )?)+\w*)\]\]\-\-\>/$data->{$1}/g;
        return $view;
}
# Check to see if there are any links or sub directories linked to a directory
sub is_webdir_removable{
	my ($this,$id) = @_;
	my @child = $this->{db}->fetch_match("webdir_topic",{parent=>$id});
	my @links = $this->{db}->fetch_match("webdir_join",{webdir_topic_id=>$id});
	if (scalar(@child) < 1 && scalar(@links) < 1){
		return 1;
	}
	return 0;
}
# Delete a record
sub delete{
	my ($this,$table,$id) = @_;
	return if !$id || !$table;
	my @tobedeleted;
	if ($table eq "webdir_topic"){
		# Only allow directory deletion if a directory has no links or sub directories.
		if ($this->is_webdir_removable($id)){
			push(@tobedeleted,{table=>"webdir_topic",id=>$id});
		}
	}elsif ($table eq "webdir_resource"){
		# Delete all join records
		my @joins = $this->{db}->fetch_match("webdir_join",{webdir_resource_id=>$id});
		foreach my $j (@joins){
			push(@tobedeleted,{table=>"webdir_join",id=>$j->{webdir_join_id}});
		}
		# Delete the main record
		push(@tobedeleted,{table=>"webdir_resource",id=>$id});
	}elsif ($table eq "webdir_join"){
		push(@tobedeleted,{table=>"webdir_join",id=>$id});
	}
	foreach my $t(@tobedeleted){
		$this->{db}->trash_key($t->{table},$t->{id});
	}
	$this->redirect("$config{server}{server}".$this->link(action=>undef,resource_id=>undef,topic_id=>undef));
	return;
}
sub is_owner{
	my ($this) = @_;
	return 1 if $this->{db}->is_admin();
	my $sectionid = $this->section_id();
	my %site = $this->{db}->fetch("section",$sectionid);
	return $this->{db}->is_manager(\%site);
}
#----------------------------------------------------------------------
1;
