#!/usr/bin/perl

package Modules::Zine::Config;

#==========================================================================
# Zine Config Module - defines default system configuration
#==========================================================================

use strict;
use ExSite::Config qw(%config &read_exsite_conf);
use Modules::BaseDCD;

use vars qw(@ISA %zconf);
@ISA = qw(Modules::BaseDCD);

%zconf = (
	  
	  # is the ZineMachine a restricted module?

	  isRestricted => 0,

	  # are Zine requests all serviced by a particular page?

	  isService => 0,

	  # linking style ("path" or "query")
	  # path:  /cgi/page.cgi/blog/What_I_Ate_Today
	  # query: /cgi/page.cgi?aid=1234

	  link_style => "query",

	  # Searching

	  search => { article => 1, comment => 0 },

	  # The length (in chars) of teaser text that is taken from 
	  # the article body.

	  teaser_length => 200,

	  # Use secure forms?

	  secure_forms => 1,

	  #================================================================
	  # COMMENTS

	  # Rules for anonymous public postings

	  allow_anonymous => 0,
	  anonymous_name => "Anonymous",
	  shrink_anonymous_comments => 1,
	  use_captcha => 1,
	  mark_unauthenticated_comments => 0,
	  unauthenticated_comment_mark => "(?)",

	  # semi-anonymous postings (logged-in user blanks the from field):
	  # If false, display their name anyway (ie. semi-anon not allowed)
	  # (This does not prevent someone from explicitly changing their
	  # name to something such as "anonymous".)

	  withhold_name => 0,

	  # Editable author name on member comments?  If true, this allows 
	  # a member to change the name they are posting under.  Posts are
	  # still connected to their real login ID, however.

	  editable_author => 1,

	  # Notify owner of parent article if a reply is posted?

	  notify => 0,

	  # Quote parent comment when replying

	  quote_parent => 1,

	  # Automatically define a subject line when starting a new thread

	  default_subject => 1,

	  # For how many seconds can you re-edit your own comments?  
	  # Times of more than 300s (5 min) are dangerous, because it can
	  # cause comments to get out of sync with their replies.

	  edit_my_comment_timeout => 0,

	  #================================================================
	  # MARKUP

	  # title heading level
	  # Article titles are <hN>...</hN> if N > 0.
	  # If level is 0 or undefined, article titles are not wrapped 
	  # in heading tags.

	  heading_level => { zine => 2,
			     article => 3,
			     comment => 0,
			     attachment => 3 },

	  # quotations are prefixed with ">" on each line;  if true, this 
	  # setting strips these prefix characters, which is handy if you 
	  # use CSS to make the quotation more obvious.

	  remove_quote_char => 0,

	  # enable emoticons in comments?
	  # (It looks cute, but there's a performance cost for making all the 
	  # image substitutions.)

	  emoticons => 1,

	  # dating method
	  #   simple - show ctime date only
	  #   creation - show "posted on " ctime date
	  #   update - show "modified on " mtime date, if mtime > ctime, 
	  #            else use ctime

	  datetype => "creation",

	  # CMS pseudo-tags
	  #   allow_cms_tags -> allow tags like <!--&Module()-->
	  #   allow_cms_links -> allow tags like <img src=[[image]]>, 
	  #                                      <a href={{1}}?>

	  allow_cms_tags => 0,
	  allow_cms_links => 0,

	  # body strut height - this forces the article body to be a certain 
	  # height, which is useful in certain pseudo-table comment layouts.

	  body_strut_size => 60,

	  #================================================================
	  # TOOLBAR

	  # Icon style
	  #   "gif" is small, colour-neutral, and cross-browser on non-white 
	  #         backgrounds
	  #   "png" is full-colur, but not IE6 compatible on non-white backgds

	  icon_style => "gif",

	  # Should we add text labels to the toolbar?

	  icon_labels => 1,

	  # tool labels

	  tool => { 
	      reply => "reply",
	      comment => "comment",
	      new => "new",
	      edit => "edit",
	      configure => "conf",
	      delete => "delete",
	      approve => "approve",
	      reject => "reject",
	      current => "current",
	      archive => "archives",
	      back => "back",
	      #back => "top",
	      rss => "rss",
	  },

	  # Set tool placement to "right" to move the toolbar to the right 
	  # side of the page, with text flowing around it.

	  tool_placement => "inline",

	  # article tools orientation (vertical or horizontal)

	  tool_orientation => "horizontal",

	  #=================================================================
	  # IMAGES & ATTACHMENTS

	  # for photo galleries, the number of images across

	  gallery_width => 4,

	  # Wrapping text around images.
	  # The maximum image width that should have text wrapped around it.

	  img_max_wrap_size => 400,

	  # The minimum body length (in bytes) to wrap around an image.

	  body_min_wrap_size => 1000,

	  # Maximum allowed size of an uploaded picture (in kilobytes)

	  max_picture_size => 48,

	  # Maximum allowed size of an uploaded picture (in pixels)
	  
	  max_picture_dim => 500,

	  # allow non-image attachments

	  allow_documents => 0,

	  # thumbnail size

	  thumbdim => 100,

	  #================================================================
	  # INDEXING & RULES

	  # maximum number of "new" articles that should be displayed in 
	  # listings of recent updates 

	  new_comments => 5,
	  new_articles => 3,

	  # how many articles should be included in RSS indexes

	  rss_limit => 10,

	  # show a "top" link in path indexes

	  show_top => 1,

	  # stop indenting sub-threads at this thread depth

	  max_thread_depth => 4,

	  # General rules (global defaults for all zine types)

	  # whatami      name/description of these items
	  # allow        whether to allow these under a zine
	  # nest         whether to allow these under themselves
	  # policy       who/how postings can be made
	  # indexlimit   maximum number to show in an index
	  # indexmin     minimum number to show in an index
	  # indexformat  how to display an article in an index
	  # indextype    how to lay out/structure the index
	  # indexsort    how to order the index
	  # indexage     maximum age of articles in an index
	  # attachments  how many attachments to solicit
	  # status       default (starting) status
	  # showdate     whether to show the posting date
	  # showauthor   whether to show the author name
	  # fields       which fields to prompt for in forms
	  # required     which fields are required in forms

	  zine => {
	      whatami => "e-zine",
	      allow => 0,
	      nest => 0,
	      showdate => 0,
	      showauthor => 0,
	      fields => ["title","subtitle","summary","about"],
	      required => ["title"],
	  },
	  article => {
	      whatami => "article",
	      allow => 1,
	      nest => 0,
	      policy => "admin",
#	      post => "admin",
	      indexlimit => 10,
	      indexmin => 3,
	      indexformat => "summary",
	      indextype => "basic",
	      indexsort => "reverse-date",
	      indexage => 60,
	      attachments => 3,
	      status => "active",
	      showdate => "ctime",
	      showauthor => 1,
	      fields => ["author","name","title","subtitle","summary","body","content_type","about","sortkey","picture","caption"],
	      required => ["title","body"],
	  },
	  comment => {
	      whatami => "comment",
	      allow => 0,
	      nest => 1,
	      policy => "member",
#	      post => "member",
	      indexlimit => 25,
	      indexmin => 5,
	      indexformat => "full",
	      indextype => "threaded",
	      indexsort => "date",
	      indexage => 60,
	      attachments => 1,
	      status => "active",
	      showdate => "ctime",
	      showauthor => 1,
	      fields => ["author","title","body"],
	      required => ["title","body"],
	  },
	  attachment => {
	      whatami => "attachment",
	      allow => 0,
	      nest => 0,
	      policy => "admin",
	      showauthor => 0,
	      showdate => 0,
	      fields => ["name","title","picture","caption"],
	      required => ["picture"],
	  },

	  # Zine-specific configurations.

	  # There are an unlimited

	  # e-zines use the defaults above

	  # Book articles are treated like chapters.  Authors are displayed
	  # at the front, but not on each chapter.  Chapters are numbered,
	  # have an explicit sort order, and do not expire.

	  book => {
	      whatami => "Book",
	      showauthor => 1,
	      fields => ["author","title","subtitle","summary","body","content_type","about","sortkey","picture","caption"],
	      zine => {
		  allow => 1,
	      },
	      article => {
		  nest => 1,
		  showauthor => 0,
		  indextype => "list",
		  indexsort => "sorted",
		  indexage => -1,
	      }
	  },

	  # Forums do not normally contain articles (but if this is disabled,
	  # we treat similarly to books).

	  forum => {
	      whatami => "Forum",
	      fields =>  ["title","subtitle","summary","sortkey"],
	      article => {
		  allow => 0,
		  indexsort => "sorted",
		  indexage => -1,
	      },
	      comment => {
		  allow => 1,
	      },
	  },

	  # Blogs have a single author, like a book, but articles are 
	  # ordered newest first, and will expire into the archive.
	  # Articles are displayed in full in the index.

	  blog => {
	      whatami => "Blog",
	      fields =>  ["title","subtitle","summary","about","picture","caption"],
	      article => {
		  whatami => "blog article",
		  indexformat => "full",
		  policy => "owner",
		  fields => ["name","title","body","about","picture","caption"],
	      },
	  },

	  # Photoblogs are similar to blogs, but we emphasize the pictures
	  # by using a paged format.

	  photoblog => {
	      whatami => "Photo Blog",
	      fields =>  ["title","subtitle","summary","about"],
	      article => {
		  whatami => "image",
		  indexformat => "full",
		  indextype => "paged",
		  attachments => -1,
		  policy => "owner",
		  fields => ["title","picture","caption","summary"],
		  required => ["title","picture"],
	      },
	  },

	  # Photoforums are similar to photblogs, except that posts 
	  # are accepted from site readers.

	  photoforum => {
	      whatami => "Photo Forum",
	      fields =>  ["title","subtitle","summary","about"],
	      article => {
		  whatami => "image",
		  policy => "member",
		  indexformat => "full",
		  indextype => "gallery",
		  attachments => -1,
		  fields => ["title","picture","caption","summary"],
		  required => ["title","picture"],
	      },
	  },

	  # Wikis have their own specialized indexing.

	  wiki => {
	      whatami => "Wiki",
	      fields =>  ["title","subtitle","summary","about","picture","caption"],
	      article => {
		  whatami => "wiki article",
		  policy => "member",
		  indextype => "wiki",
		  indexformat => "full",
		  indexsort => "date",
	      },
	  },

	  #===============================================================
	  # MISCELLANEOUS FORMATTING

	  # fields to prompt for on article forms

	  # comment form labels

	  comment_label => {
	      author => "From",
	      body => "Comment",
	      title => "Subject",
	  },

	  # sort rules

	  sort_methods => {
	      sorted => "sortkey",
	      date => "ctime",
	      "reverse-date" => {"ctime"=>"desc"},
	      "flip-flop" => "ctime",
	      "reverse-flip-flop" => {"ctime"=>"desc"},
	  },

	  # search weights 

	  search_weight => {
	      title=>3,
	      body=>1,
	      author=>2,
	      name=>3,
	      subtitle=>2,
	      summary=>1,
	      about=>1,
	      caption=>1,
	  },

	  # list of supported emoticons

	  emoticon_re => ['>:-?\)', '(:-?\)|\(-?:)', '(:-?\(|\)-?:)', 
			  ':-?D', ':-?[Oo]', ';-?[\)D]', ':-?P ' ],

	  # emoticon images

	  emoticon_img => {
	      '(:-?\)|\(-?:)' => "emoticon_smile.png",
	      '(:-?\(|\)-?:)' => "emoticon_unhappy.png",
	      ':-?D' => "emoticon_grin.png",
	      ':-?[Oo]' => "emoticon_surprised.png",
	      ';-?[\)D]' => "emoticon_wink.png",
	      ':-?P ' => "emoticon_tongue.png",
	      '>:-?\)' => "emoticon_evilgrin.png",
	  },

	  # flag article status

	  status_icon => {
	      inactive=>"lock.png",
	      active=>"accept.png",
	      pending=>"pending.gif",
	      rejected=>"stop.png",
	      archived=>"calendar.png",
	  },

	  );

# create a basic Zine object

sub new {
    my $this = shift;
    my $article = shift;
    my $opt = shift;

    my $obj = {};
    my $class = ref($this) || $this;
    bless $obj, $class;

    # setup base object 
    $obj->initialize_object;
    foreach my $key (keys %$opt) { $obj->{$key} = $opt->{$key}; }
    $obj->{save} = {
	article => [],
	article_id => [],
	reference => [],
    };

    # default configuration
    $config{$this->module_name} = \%zconf;
    # load site-specific configurations
    $obj->read_conf;
    # include configurations in the object, so we don't need to know our
    # module name
    $obj->{config} = $config{$this->module_name};

    # automatically read the default input source
    $obj->read;

    # misc setup
    $obj->setup($article,$opt);

    return $obj;
}

# dummy setup
sub setup {
    my ($this,$article,$opt) = @_;
    return;
}

# confrule: find and return a configuration rule
#
# Configuration rules apply to certain classes of articles.  The
# default rule is CLASS.RULE (eg. article.indextype - which defines
# which index type to use when listing articles).  There are also 
# zine-specific rules of the form ZINE.CLASS.RULE (eg. 
# blog.article.indextype - which defines which index type to use when 
# listing articles under a blog).
#
# If the class is "zine", then we assume we are looking for the
# ZINE.RULE, not ZINE.zine.RULE.
#
# Given ZINE (optional) and CLASS (required), we return the appropriate
# rule to follow.  The Zine-specific rule has precedence over the 
# default rule.
#
# Some (but not all) rules can also be specified in the data records.
# To use those rule preferentially over the config rules, use
# Modules::Zine::Base::rule()
#
# $rule is typically one of:
# indexlimit: maximum number of articles to index
# indexmin: minimum number of articles to index
# indexformat: format of each article in an index
# indextype: layout of the whole index
# indexsort: order of the articles in the index
# indexage: maximum age (in days) of articles in the index
# attachments: number of accepted attachments when posting
# status: default status of newly posted articles

sub confrule { 
    my ($this,$rule,$class,$zinetype) = @_;

    # check for a zine-specific rule in the config file
    if ($zinetype) {
	if ($class eq "zine" || ! $class) {
	    if (exists $this->{config}{$zinetype}{$rule}) {
		return $this->{config}{$zinetype}{$rule};
	    }
	}
	else {
	    if (exists $this->{config}{$zinetype}{$class}{$rule}) {
		return $this->{config}{$zinetype}{$class}{$rule};
	    }
	}
    }
    
    # check for a general rule in the config file
    if ($class) {
	if (exists $this->{config}{$class}{$rule}) {
	    # there is a general rule set in the config file
	    return $this->{config}{$class}{$rule};
	}
    }
    
    # no rule found
    return undef;
}

1;

	  
