#!/usr/bin/perl

package Modules::Zine::IndexArchive;

=pod

=head1 Modules::Zine::IndexArchive

This is a plug-in indexing engine to display article indexes broken
out by date.  It ignores age, limit, and archive settings, and shows 
the entire index, broken out by month and year (oldest last).

TODO: for long archives, we can break it down into individual years or
months.

=cut

use strict;
use ExSite::Config;
use ExSite::Tree;
use Modules::Zine::Article;

use vars qw(@ISA);
@ISA = qw(Modules::Zine::Article);

sub index {
    my ($this,%opt) = @_;
    my $stat = $this->handler("ZineIndexArchive",%opt);
    return $stat if ($stat);

    # get all the articles in the index
    my @articles = $this->fetch_index(%opt);

    return (@articles > 0) ?
	$this->index_archive(\%opt,@articles) :
	undef;                          # show nothing in empty archives
	#"<p>Archive is empty.</p>";    # show a message
}

sub index_archive {
    my ($this,$opt,@articles) = @_;
    my $out;
    $this->save;
    my ($month,$year,$month2,$year2);
    my ($time,$tlist);

    # which date should we compare to?
    my $dtype = $this->{config}{datetype} eq "update" ? "mtime" : "ctime";

    foreach my $art (@articles) {
	$this->set($art);
	next if ($this->type ne $opt->{mode});
	$time = new ExSite::Time($this->{article}{$dtype},"sql_timestamp");
	$tlist = $time->write("raw");
	$year2 = $tlist->[0];
	$month2 = $tlist->[1];
	if ($year2 != $year || $month2 != $month) {
	    $year = $year2;
	    $month = $month2;
	    $out .= "<h2>".$time->write("month_long")." ".
		$time->write("year")."</h2>\n";
	}
	if ($art->{status} eq "active" || $this->moderator) {
	    $out .= "<div class=\"ZineItem\">\n".$this->show_info()."</div>\n";
	}
    }

    $this->restore;
    return $out;
}

sub fetch_index {
    my ($this,%opt) = @_;

    my @articles;
    my $start = $this->{article_id};
    if ($start) {
	my $match = {};
	my $class = $this->class();

	if ($class eq "zine") {
	    $match->{reference} = $start;
	}
	elsif ($class eq "article") {
	    if ($opt{layout} eq "paged" && $opt{mode} ne "comment") {
		$start = $this->{article}{reference};
		$match->{reference} = $start;
	    }
	    elsif ($opt{mode} eq "comment") {
		$match->{reference} = $start;
	    }
	    else {
		$match->{reference} = $this->{article}{reference};
	    }
	}
	else {
	    $match->{reference} = $this->{article}{reference};
	}
	my $tree = $this->fetch_tree($start,$match);
	@articles = $this->sort("reverse-date",$tree->collapse($start));
    }
    $this->{nart} = scalar @articles;
    return wantarray ? @articles : \@articles;
}

sub archive_count {
    return -1;
}

sub article_count {
    my $this = shift;
    return $this->{nart};
}

1;

